{

GBP_FLASH_T9

Testprogram 9 for Gam_Bo_Prop

Testing FLASH

You have to do calibration with testprogram 8 (GBP_FLASH_T8) at first.

'Gam_Bo_Prop' is the short name of 'game board propeller'
'Gambo' is also the name given to an unidentified large marine animal

based upon Propeller C3 Onboard FLASH memory demo -  c3_flash_demo_010.spin

modified by GKR



 Propeller C3 Onboard FLASH memory demo -  c3_flash_demo_010.spin
 AUTHOR: Andre' LaMothe
 LAST MODIFIED: 9/3/10
 VERSION: 1.0
 LICENSE: MIT
 
 COMMENTS: This demo illustrates the use of the on board 1MB SPI serial FLASH
 memory module. The FLASH memory is manufactured by Atmel part #AT26DF081A-SSU.
 The datasheet for the chip can be found here:

 http://www.atmel.com/dyn/products/product_card.asp?part_id=3822

 The chip runs up to 70 Mhz on the SPI bus, has 256 byte pages and is compatible
 with the JEDEC spec for FLASH memory and pinouts. In other words, if you have
 written a FLASH driver for say a Microchip, or ST part then chances are it will
 take moments to port it to this chip.

 That said, FLASH chips have HUGE functionality and writing a generic driver that
 supports all the functions would be a waste. Therefore, I have provided a "starter"
 API here to get you going, so you can write, read, erase the chip and start
 experimenting. I am sure there are numerous serial FLASH drivers you can find
 for the chip and even some in SPIN! Here's a list of the API functions:

 PUB Flash_Read( flash_start_addr, buffer_ptr, num_bytes)  - Reads bytes from flash into a buffer. 
 PUB Flash_Write( flash_start_addr, buffer_ptr, num_bytes) - Writes bytes to flash from buffer. 
 PUB Flash_Erase( block ) - Erases a block of the flash. 
 PUB Flash_Erase_Chip( wait ) - Erases the entire chip.
 PUB Flash_Close          - Closes the flash (not much now, but this is a good place to de-allocate things in the future).       
 PUB Flash_Open           - Opens the flash (again not much now, but this is a good place to allocate things in the future). 
 PUB Flash_Write_Status ( status ) - Writes the STATUS register of the flash memory. 
 PUB Flash_Read_Status             - Reads the STATUS register of the flash memory.
 
 These functions more or less give you access to the FLASH memory and are based on
 a handful of the FLASH commands (read the datasheet to learn more). 

 Additionally, this demo has a number of defines/constants to help deal with the
 chip, so pay attention to those if you want to pull the source from this file to
 build an application then you will need those constants. Additionally, there is
 data structure that holds starting addresses of "sectors". The Atmel chip like
 many others has a number of quirks, one of them is that instead of having same
 sized sectors, it has different sized sectors at differnet addresses, so we need
 a list of these since the code to generate them procedurally would be 10x longer
 than a simple table. Thus, you need the data structure "sector_addresses" in the
 DAT section below.

 The demo program is a bit complex since it basically gives you a conduit to the FLASH
 memory, so you can read/write data, etc. So, if you know what an assembly language
 "monitor" is, it feels like that -- if you don't, in other words, all you do is read
 and write memory and type a lot :) However, you can easily use the API to save data
 from a mic or something else plugged into the A/D port, then play it back, or whatever.
 Point is, this is a 1 MegaByte FLASH -- a good place to store data that STAYS with the
 C3 and can't be pulled out like the SD card.   
  
 The action all occurs in the main() function, below the main() are a number of "utility"
 functions  included in each demo for reference. If you wish you can break them out into an
 external spin module. They are mostly for parsing and various random functions needed
 in each demo. The only important set of functions for the C3 are the "SPI" bus control
 functions, pay close attention to those since they control the selection of the various
 SPI devices on the bus. 
 
 REQUIRES: PS/2 keyboard plugged in and A/V port connected to NTSC TV. 
   
}

CON

' PROP CHIP / SYSTEM DEFINES //////////////////////////////////////////////////
  ' These settings are for 10 MHZ XTALS
  _clkmode = xtal1 + pll8x          ' enable external clock and pll times 8
  _xinfreq = 10_000_000             ' set frequency to 10 MHZ

  CLOCKS_PER_MICROSECOND = 10*8     ' simple xin*pll / 1_000_000
  CLOCKS_PER_MILLISECOND = 10000*8  ' simple xin*pll / 1_000

  _stack   = 125                    ' accomodate display memory and stack

' Gam_Bo_Prop PIN MAP DEFINES

  ' SPI pins
{
  PIN#s direct implemented in subroutine
  SPI_SEL0       = 0  ' used to select SPI channel
  SPI_SEL1       = 1  ' used to select SPI channel
  SPI_SEL2       = 2  ' used to select SPI channel
}

  SPI_MOSI       = 9  ' SPI master out serial in to slave
  SPI_MISO       = 10 ' SPI master in serial out from slave, 10 for good C3, change to 7 for BROKEN C3 with YELLOW jumper.
  SPI_SCK        = 11 ' SPI clock from master to all slaves

  ' SPI channels defines
  SPI_CHAN_NULL          = 0 ' NULL channel, disables all on/off board devices.
  SPI_CHAN_SRAM0         = 1 ' 32K SRAM Bank 0.
  SPI_CHAN_SRAM1         = 2 ' 32K SRAM Bank 1.
'  SPI_CHAN_FLASH0        = 3 ' 2MB FLASH Memory.Bank 0
'  SPI_CHAN_FLASH1        = 4 ' 2MB FLASH Memory Bank 1
  SPI_CHAN_SD            = 5 ' SD Card./ SD-ModulChannel 0
  SPI_CHAN_SDM1          = 6 ' SD-Modul Channel 1.
  SPI_CHAN_SDM2          = 7 ' SD-Modul Channel 2.

  ' Detection of SD-Card or SD-Modul
  WP            = 6 ' Detect WP used with CDET to determine, if a SD-Card or SD-Modul is inserted
  CD            = 7 ' Detect CD (Card detect)



  ' Audio stereo output pin define (PWM)
  A_RIGHT       = 4
  A_LEFT        = 5
{
  PIN#s direct implemented in subroutine
  KEY_A         = 12
  KEY_B         = 13
  KEY_C         = 14
  KEY_D         = 15
}

  ' LCD-Display
  LCD_LED       = 8
  LCD_DO        = 18
  LCD_SCK       = 19
  LCD_DI        = 20
  LCD_CS        = 21
  LCD_RES       = 22

  ' Touchscreen controller
  TO_DO         = 23
  TO_SCK        = 24
  TO_DI         = 25
  TO_CS         = 26

  ' I2C pin defines (standard Prop pins)
  SCL           = 28
  SDA           = 29

  ' TX/RX pin defines (standard Prop pins)
  RXD           = 30
  TXD           = 31

  ' LCD-Color
  bcolor        = $0000FF
  fcolor        = $FFFF00


{
  'Unused and unconnected Pins
  P3            = 3
  P16           = 16
  P17           = 17
  P27           = 27
}

  ' ASCII codes for ease of parser development
  ASCII_A      = 65
  ASCII_B      = 66
  ASCII_C      = 67
  ASCII_D      = 68
  ASCII_E      = 69
  ASCII_F      = 70
  ASCII_G      = 71
  ASCII_H      = 72
  ASCII_O      = 79  
  ASCII_P      = 80
  ASCII_Z      = 90
  ASCII_0      = 48
  ASCII_9      = 57
  ASCII_LEFT   = $C0
  ASCII_RIGHT  = $C1
  ASCII_UP     = $C2
  ASCII_DOWN   = $C3 
  ASCII_BS     = $C8 ' backspace
  ASCII_DEL    = $C9 ' delete
  ASCII_LF     = $0A ' line feed 
  ASCII_CR     = $0D ' carriage return
  ASCII_ESC    = $CB ' escape
  ASCII_HEX    = $24 ' $ for hex
  ASCII_BIN    = $25 ' % for binary
  ASCII_LB     = $5B ' [ 
  ASCII_SEMI   = $3A ' ; 
  ASCII_EQUALS = $3D ' = 
  ASCII_PERIOD = $2E ' .
  ASCII_COMMA  = $2C ' ,
  ASCII_SHARP  = $23 ' #
  ASCII_NULL   = $00 ' null character
  ASCII_SPACE  = $20 ' space
  ASCII_TAB    = $09 ' horizontal tab

  ' null pointer, null character
  NULL         = 0  
' FLASH MEMORY DEFINES

' defines for the proper FLASH installed on C3 4MB, 8MB, 16MB
' 4Mb (512K Bytes)
AT26F004        = 4  ' numeric codes for flash version
AT25DF041A      = 4

' 8Mb (1M Bytes)
AT26DF081A      = 8
AT25DF081A      = 8

' 16 Mb (2M Bytes)
AT25DF161       = 16

' set type of FLASH ROM currently installed in C3 (8MBit) for conditional compilation of code -- not supported in SPIN, so manually done below :( 
FLASH_ROM             = AT25DF161
NUM_FLASH_ROM_SECTORS = 16 ' 11 for 4MB, 19 for 8MB,16  for 16 MB must be set manually due to lack of conditional compilation


' Atmel 2F004/8xxx series basic commands
WRITE_ENABLE     = $06
WRITE_DISABLE    = $04
READ_STATUS      = $05
WRITE_STATUS     = $01
READ_DATA        = $03

SEQ_PROGRAM      = $AF  
BYTE_PROGRAM     = $02 

BLOCK_ERASE4     = $20
BLOCK_ERASE32    = $52
BLOCK_ERASE64    = $D8

SECTOR_ERASE     = $20    

CHIP_ERASE       = $60 
JEDEC_ID         = $9F

PROTECT_SECTOR   = $36 
UNPROTECT_SECTOR = $39 

' custom commands we make up, not part of chip spec, just used here, start at code 0xF0
UNPROTECT_CHIP   = $F0



VAR

  ' generic globals to make demo writing easy
  long g_data, g_data2, g_status    
  long g_data32, g_data16, g_spi_result
  long g_key, g_temp1, g_temp2, g_temp3, g_index, g_length
  long g_tempx, g_tempy 
  ' this buffer is used as the read/write buffer for the FLASH, so you can't read or write
  ' more bytes than the size of this buffer
  byte g_sbuffer[4096]

  long SPI_channel'Channel # which will be selected
  long keycode    'code from game-keyboard (on board 8 buttons)
  long SD_Status
  long TSkeycode  'code from touchscreen keyboard

DAT

'  conditional compilation for FLASH memory variants not supported, so just comment out either declaration

' #if (FLASH_ROM == AT26F004)

'// we could use PGM memory for these and or do it algorithmically, but this is cleaner for now
'// 11 sectors [64K, 64K, 64K, 64K, 64K, 64K, 64K, 32K, 8K, 8k, 16K ]
'//            |<------- (7) 64K sectors ------->|  1      2     1 

{
    ' 11 sectors in 4MB FLASH
    sector_addresses long $00000   ' 64K sector 0
                     long $10000   ' 64K sector 1
                     long $20000   ' 64K sector 2
                     long $30000   ' 64K sector 3
                     long $40000   ' 64K sector 4
                     long $50000   ' 64K sector 5
                     long $60000   ' 64K sector 6
                     long $70000   ' 32K sector 7
                     long $78000   ' 8K sector 8  
                     long $7A000   ' 8K sector 9
                     long $7D000   ' 8K sector 10


' #if (FLASH_ROM == AT26DF081A)

' we could use PGM memory for these and or do it algorithmically, but this is cleaner for now
' 19 sectors [64K, 64K, 64K, ..., 64K, 64K, 64K, 64K, 16K, 8K, 8k, 32K ]
'            |<----------- (15) 64K sectors -------->| 1     2     1 

    ' 19 sectors in 8MB FLASH
    sector_addresses long $00000   ' 64K sector 0
                     long $10000   ' 64K sector 1
                     long $20000   ' 64K sector 2
                     long $30000   ' 64K sector 3
                     long $40000   ' 64K sector 4
                     long $50000   ' 64K sector 5
                     long $60000   ' 64K sector 6
                     long $70000   ' 64K sector 7 
                     long $80000   ' 64K sector 8
                     long $90000   ' 64K sector 9
                     long $A0000   ' 64K sector 10
                     long $B0000   ' 64K sector 11
                     long $C0000   ' 64K sector 12
                     long $D0000   ' 64K sector 13
                     long $E0000   ' 64K sector 14
                          
                     long $F0000   ' 16K sector 15
                     long $F4000   '  8K sector 16
                     long $F6000   '  8K sector 17
                          
                     long $F8000    ' 32K sector 18
}

' we could use PGM memory for these and or do it algorithmically, but this is cleaner for now
' 32 sectors [64K,64K, 64K,..... ...,. 64K, 64K, 64K ]
'            |<----------- (32) 64K sectors -------->|

    ' 32 sectors in 16MB FLASH
    sector_addresses long $000000   ' 64K sector 0
                     long $010000   ' 64K sector 1
                     long $020000   ' 64K sector 2
                     long $030000   ' 64K sector 3
                     long $040000   ' 64K sector 4
                     long $050000   ' 64K sector 5
                     long $060000   ' 64K sector 6
                     long $070000   ' 64K sector 7
                     long $080000   ' 64K sector 8
                     long $090000   ' 64K sector 9
                     long $0A0000   ' 64K sector 10
                     long $0B0000   ' 64K sector 11
                     long $0C0000   ' 64K sector 12
                     long $0D0000   ' 64K sector 13
                     long $0E0000   ' 64K sector 14
                     long $0F0000   ' 64K sector 15
                     long $100000   ' 64K sector 16
                     long $110000   ' 64K sector 17
                     long $120000   ' 64K sector 18
                     long $130000   ' 64K sector 19
                     long $140000   ' 64K sector 20
                     long $150000   ' 64K sector 21
                     long $160000   ' 64K sector 22
                     long $170000   ' 64K sector 23
                     long $180000   ' 64K sector 24
                     long $190000   ' 64K sector 25
                     long $1A0000   ' 64K sector 26
                     long $1B0000   ' 64K sector 27
                     long $1C0000   ' 64K sector 28
                     long $1D0000   ' 64K sector 29
                     long $1E0000   ' 64K sector 30
                     long $1F0000   ' 64K sector 31


OBJ

  ' current drivers used in this version of SPI interface
  key   : "GBP_gkey.spin"             ' keyboard driver
  lcd   : "GBP_MIO283QT2.spin"        ' LCD driver
  led   : "GBP_BGLED.spin"
  ts    : "GBP_ADS7846.spin"          ' touchscreen driver
  rtc   : "DS1307_RTCEngine.spin"

PUB Main : status
  ' initialize the LCD-Display
  lcd.start(LCD_CS, LCD_SCK, LCD_DI, LCD_RES)

  'RTC-Engine needed for RTC-Ram (calibration of touchscreen)
  rtc.RTCEngineStart(SDA, SCL, -1)

  ' Backlight on
  status := led.start(30)

  ' initialize the Keyboard
  status := key.start

  ts.SPI_Init

  ' initialize SPI IO (eventhough, we aren't using it in this demo at all, good habit to set it up)
  SPI_Init
  
  ' begin test loop
  ' clear screen
'  Delay_MS(60000 )
  lcd.clear

  repeat
    Flash_Demo

         
' end Main


PUB Flash_Demo | i,str ',TSkeycode

  ' FLASH MEMORY DEMO

  ' open the flash memory up (unprotect it)

  'Flash_Open

  'Read calibration from RTC-Ram
  ts.readCal
  repeat
    ' draw menu
    lcd.Text( 3, 1, string ("Gam_Bo_Prop Flash Memory Test"))
    lcd.Text( 3, 3, string ("Key 1 = Select channel"))
    lcd.Text( 3, 4, string ("Key 2 = Read ID"))
    lcd.Text( 3, 5, string ("Key 3 = Read STATUS register"))
    lcd.Text( 3, 6, string ("Key 4 = Write STATUS reg. (not used)"))
    lcd.Text( 3, 7, string ("Key 5 = Write [start,value,numbytes]"))
    lcd.Text( 3, 8, string ("Key 6 = Read  [start,numbytes]"))
    lcd.Text( 3, 9, string ("Key 7 = Erase chip"))
    lcd.Text( 3, 10, string ("Key 8 = Repeat main menu"))
    lcd.Text( 3, 11, string ("First select channel with key 1"))
    lcd.Text( 3, 12, string ("Press key 1 to 8"))

    ' wait for keypress (button 1 to 8)
    keycode := key.waitforkey

    ' what is user requesting?
    case keycode
      1:
        lcd.clear
        lcd.setX(3)
        lcd.pstring(string ("Gam_Bo_Prop Flash Memory Test"))
        lcd.crlf
        SD_Status := SDM_Detect
        if SD_Status <> 2
          'lcd.pstring(STRING (itoa(SD_Status)))
          lcd.hex(SD_Status, 2)
          lcd.crlf
          lcd.pstring(string ("Select channel 3,4"))
          lcd.crlf
          lcd.pstring(string ("Push key :3,4"))
          keycode := key.waitforkey
          case keycode
            3 : SPI_channel := 3
            4 : SPI_channel := 4
            other :  keycode := 0
                  SPI_channel:= 0
                  lcd.clear
                  quit
        else
          lcd.hex(SD_Status, 2)
          lcd.crlf
          lcd.pstring(string ("Select channel 3,4 or 5,6,7 on SD-Modul"))
          lcd.crlf
          lcd.pstring(string ("Push key :3,4 or 5,6,7"))
          keycode := key.waitforkey
          case keycode
            3 : SPI_channel := 3
            4 : SPI_channel := 4
            5 : SPI_channel := 5
            6 : SPI_channel := 6
            7 : SPI_channel := 7
            other :  keycode := 0
                  SPI_channel:= 0
                  lcd.clear
                  quit
        Flash_Open
        lcd.crlf
        lcd.pstring(string ("Selected SPI-channel : "))
        lcd.dec (SPI_channel)
        lcd.crlf
        lcd.pstring(string ("Push any key to return to main menu..."))
        keycode := key.waitforkey
        keycode := 0
        lcd.clear

      2:
        lcd.clear
        lcd.setX(3)
        lcd.pstring(string ("Gam_Bo_Prop Flash Memory Test"))
        lcd.crlf
        lcd.crlf
        lcd.pstring(string ("Read the ID of selected flash memory"))
        lcd.crlf
        lcd.pstring(string ("SPI-channel : "))
        lcd.dec (SPI_channel)
        lcd.crlf
        Flash_ID (@g_sbuffer, 4)
        lcd.pstring(string ("JEDEC-ID    : "))
        repeat i from 0 to 3
          ' print Jedec-ID
          lcd.hex (g_sbuffer[i], 2 )
        lcd.crlf
        lcd.pstring(string ("Push any key to return to main menu..."))
        keycode := key.waitforkey
        keycode := 0
        lcd.clear


      3:
        lcd.clear
        lcd.setX(3)
        lcd.pstring(string ("Gam_Bo_Prop Flash Memory Test"))
        g_temp1 := Flash_Read_Status
        lcd.crlf
        lcd.crlf
        lcd.pstring(string ("Read Status Register") )
        lcd.crlf
        lcd.pstring(string ("Status Register : ") )
        lcd.dec( g_temp1 )
        lcd.crlf
        lcd.pstring(string ("Push any key to return to main menu..."))
        keycode := key.waitforkey
        keycode := 0
        lcd.clear

      4:

      5:
        lcd.clear
        drawTSKeyboard
        lcd.setX(1)
        lcd.setY(1)
        lcd.pstring(string ("Gam_Bo_Prop Flash Memory Test"))
        lcd.crlf
        lcd.pstring(string ("dec input, hex output") )
        lcd.crlf
        lcd.pstring(string ("Write to flash memory"))
        lcd.crlf
        lcd.pstring(string ("Start address?") )

        ' get string from user
        Get_String(@g_sbuffer, 8 )

        ' convert to integer
        g_temp1 := atoi2(@g_sbuffer, 8)

        lcd.crlf
        lcd.pstring( string ("Value to write?") )

       ' get string from user
        Get_String(@g_sbuffer, 8 )

       ' convert to integer
        g_temp2 := atoi2(@g_sbuffer, 8)

        lcd.crlf
        lcd.pstring(string ("Number of bytes to write?") )

        ' get string from user
        Get_String(@g_sbuffer, 8 )

        ' convert to integer
        g_temp3 := atoi2(@g_sbuffer, 8)

        ' fill buffer with value
        bytefill ( @g_sbuffer, g_temp2, g_temp3)

        ' write buffer to flash
        Flash_Write( g_temp1, @g_sbuffer, g_temp3 )

        lcd.crlf
        lcd.pstring(string ("Push any key to return to main menu..."))
        keycode := key.waitforkey
        keycode := 0
        lcd.clear

      6:
        lcd.clear
        drawTSKeyboard
        lcd.setX(1)
        lcd.setY(1)
        lcd.pstring(string ("Gam_Bo_Prop Flash Memory Test"))
        lcd.crlf
        lcd.pstring(string ("dec input, hex output") )
        lcd.crlf
        lcd.pstring(string ("Read from flash memory"))
        lcd.crlf
        lcd.pstring(string ("Start address?") )

        ' get string from user
        Get_String(@g_sbuffer, 8 )

        ' convert to integer
        g_temp1 := atoi2(@g_sbuffer, 8)

        lcd.crlf
        lcd.pstring(string ("Number of bytes to read?") )

        ' get string from user
        Get_String(@g_sbuffer, 8 )

        ' convert to integer
        g_temp3 := atoi2(@g_sbuffer, 8)

        ' read flash into buffer
        Flash_Read( g_temp1, @g_sbuffer, g_temp3 )

        ' print buffer to screen
        lcd.crlf
        lcd.pstring(string ("Bytes read:") )
        lcd.clear
        repeat g_index from 0 to g_temp3-1
          ' print starting address to left every 8 bytes
          if ((g_index // 8) == 0)
            lcd.crlf
            lcd.hex(g_index + g_temp1, 6 )
            lcd.pstring( string (":") )

          ' print byte
          lcd.hex( g_sbuffer [ g_index ], 2 )
          lcd.out( "," )

        lcd.crlf
        lcd.pstring(string ("Push any key to return to main menu..."))
        keycode := key.waitforkey
        keycode := 0
        lcd.clear

    
     7: 'erase chip
        lcd.clear
        lcd.setX(3)
        lcd.pstring(string ("Gam_Bo_Prop Flash Memory Test"))
        lcd.crlf
        lcd.crlf
        lcd.pstring(string ("Erase Flash Memory on...."))
        lcd.crlf
        lcd.pstring(string ("SPI-channel : "))
        lcd.dec (SPI_channel)
        lcd.crlf
        lcd.crlf
        lcd.pstring(string ("Erasing chip..."))
        Flash_Erase_Chip(1)
        lcd.crlf
        lcd.pstring(string ("Erasure complete!"))
        lcd.crlf
        lcd.pstring(string ("Push any key to return to main menu..."))
        keycode := key.waitforkey
        keycode := 0
        lcd.clear

     ' return to caller
     8: 'return
        lcd.clear

' end Flash_Test

PUB drawTSKeyboard | i, a
' Drawing a touchscreen keyboard
  lcd.Text( 1, 15, string ("                             BS   Enter "))
  lcd.Text( 1, 17, string (" 1 2 3 4 5 6 7 8 9 0 Q W E R T Z U I O P"))
  lcd.Text( 1, 19, string (" A S D F G H J K L Y X C V B N M . - _  "))
  lcd.drawLine(0, 162, 319, 162, $ffff00)
  lcd.drawLine(0, 186, 319, 186, $ffff00)
  lcd.drawLine(0, 210, 319, 210, $ffff00)
  lcd.drawLine(0, 234, 319, 234, $ffff00)
  lcd.drawLine(227,162,227,186,$ffff00)
  lcd.drawLine(259,162,259,186,$ffff00)
  repeat i from 1 to 19
    a := i*16+3
    lcd.drawLine(a, 186, a, 234, $ffff00)

PUB getXS
' Get x Position in screen pixel
  return ts.calX(ts.getX,ts.getY)

PUB getYS
' Get y Position in screen pixel
  return ts.calY(ts.getX,ts.getY)

PUB getTSkey | x, y, code, v
code := 0
repeat while code == 0
  y := getYS
  x := getXS
  If (y>186)and(y<210)
    if (x>  0)and(x< 19)
      code := 49
    elseif (x> 19)and(x< 35)
      code := 50
    elseif (x> 35)and(x< 51)
      code := 51
    elseif (x> 51)and(x< 67)
      code := 52
    elseif (x> 67)and(x< 83)
      code := 53
    elseif (x> 83)and(x< 99)
      code := 54
    elseif (x> 99)and(x<115)
      code := 55
    elseif (x>115)and(x<131)
      code := 56
    elseif (x>131)and(x<147)
      code := 57
    elseif (x>147)and(x<163)
      code := 48
    elseif (x>163)and(x<179)
      code := 81
    elseif (x>179)and(x<195)
      code := 87
    elseif (x>195)and(x<211)
      code := 69
    elseif (x>211)and(x<227)
      code := 82
    elseif (x>227)and(x<243)
      code := 84
    elseif (x>243)and(x<259)
      code := 90
    elseif (x>259)and(x<275)
      code := 85
    elseif (x>275)and(x<291)
      code := 73
    elseif (x>291)and(x<307)
      code := 79
    elseif (x>307)and(x<319)
      code := 80
    else
      code := 0

  elseif (y > 210) and (y < 234)
    if (x>  0)and(x< 19)
      code := 65
    elseif (x> 19)and(x< 35)
      code := 83
    elseif (x> 35)and(x< 51)
      code := 68
    elseif (x> 51)and(x< 67)
      code := 70
    elseif (x> 67)and(x< 83)
      code := 71
    elseif (x> 83)and(x< 99)
      code := 72
    elseif (x> 99)and(x<115)
      code := 74
    elseif (x>115)and(x<131)
      code := 75
    elseif (x>131)and(x<147)
      code := 76
    elseif (x>147)and(x<163)
      code := 89
    elseif (x>163)and(x<179)
      code := 88
    elseif (x>179)and(x<195)
      code := 67
    elseif (x>195)and(x<211)
      code := 86
    elseif (x>211)and(x<227)
      code := 66
    elseif (x>227)and(x<243)
      code := 78
    elseif (x>243)and(x<259)
      code := 77
    elseif (x>259)and(x<275)
      code := 46
    elseif (x>275)and(x<291)
      code := 45
    elseif (x>291)and(x<307)
      code := 95
    elseif (x>307)and(x<319)
      code := 32
    else
      code := 0
  elseif (y > 162) and (y < 186)
    if (x>  227)and(x< 259)
      code := 8
    elseif (x>259)and(x<319)
      code := 13
    else
      code := 0
  delay_MS(500)
return code

PUB Flash_Read_Status | _spi_data
{{
Reads the FLASH memory status register

PARMS: None.
RETURNS: status register value
}}

  '// set CS to SPI select channel 3 (FLASH)
  SPI_Select_Channel( SPI_channel )
  _spi_data := SPI_Write_Read( 8, READ_STATUS, $FF )

  ' now read data returned
  _spi_data := SPI_Write_Read( 8, $FF, $FF )
   
  '// disable SPI interface
  '// set CS to SPI select channel 0 (null)
  SPI_Select_Channel( 0 )

  ' return results
  return ( _spi_data )   

' // end Flash_Read_Status


PUB Flash_Write_Status ( status ) | _spi_data
{{
Writes the FLASH memory status register

PARMS:

status - value to update the status register with

RETURNS: None.
}}

  '// set CS to SPI select channel 3 (FLASH)
  SPI_Select_Channel( SPI_channel)
  _spi_data := SPI_Write_Read( 8, WRITE_STATUS, $FF )
  _spi_data := SPI_Write_Read( 8, status, $FF )  
   
  '// disable SPI interface
  '// set CS to SPI select channel 0 (null)
  SPI_Select_Channel( 0 )

  ' return success for now
  return ( 1 )   

' // end Flash_Write_Status

PUB SDM_Detect | Status, writep, cardd
'This function detect a SD-Card or Modul in the SD-Connector
'and print the SD-Status in Line 1 Position 33 to 40

  dira[WP]~
  dira[CD]~
  writep := ina[WP]
  cardd  := ina[CD]
  if writep == 0 and cardd == 1
    lcd.Message(33, 1, string("SD-Modul"))
    return Status := 2
  elseif (writep == 1) and (cardd == 1)
    lcd.Message(33, 1, string("empty   "))
    return Status := 0
  else
    lcd.Message(33, 1, string("SD-Card "))
    return SD_Status := 1

PUB Flash_Open | _index, _spi_data, _flash_start_addr
{{

This functions "opens" the FLASH memory simply by unprotecting all sectors starting at address 0

PARMS: None.
RETURNS: None.

}}


  ' use global unprotect to unprotect entire chip at once, write "0" to bits 2,3,4,5
  SPI_Select_Channel( SPI_channel )
  _spi_data := SPI_Write_Read( 8, WRITE_ENABLE, $FF )
  SPI_Select_Channel( 0 )

  Flash_Write_Status ( Flash_Read_Status & %11_0000_11 )
  Delay_MS( 10 )   
  ' // return success always
  return( 1 )

{
  

  ' this code is used to iterate thru all sectors manually and have more control
  ' iterate thru all sectors and unprotect them
  repeat _index from 0 to NUM_FLASH_ROM_SECTORS-1

    '// STEP 1: enable writing to chip
    '// enable SPI interface

    '// set CS to SPI select channel 3 (FLASH)
    SPI_Select_Channel( SPI_CHAN_FLASH )
    _spi_data := SPI_Write_Read( 8, WRITE_ENABLE, $FF )

    '// disable SPI interface
    '// set CS to SPI select channel 0 (null)
    SPI_Select_Channel( 0 )

    Delay_MS( 10 )

    '// unprotect sector

    '// enable SPI interface

    '// set CS to SPI select channel 3 (FLASH)
    SPI_Select_Channel( SPI_CHAN_FLASH )
    _spi_data := SPI_Write_Read( 8, UNPROTECT_SECTOR, $FF )

    '// get sector address
    _flash_start_addr := sector_addresses[ _index ]

    '// write sector address, there are only a handful of sectors since each is 4-64K
    _spi_data := SPI_Write_Read( 8, _flash_start_addr >> 16, $FF )
    _spi_data := SPI_Write_Read( 8, _flash_start_addr >> 8, $FF )
    _spi_data := SPI_Write_Read( 8, _flash_start_addr >> 0, $FF )

    '// disable SPI interface
    '// set CS to SPI select channel 0 (null)
    SPI_Select_Channel( 0 )

    ' delay 10
    Delay_MS( 10 ) 

    ' // end for index

  ' // return success always
  return( 1 )
 }

' // end Flash_Open


PUB Flash_Close | _index, _spi_data, _flash_start_addr      
{{
This function "closes" the FLASH by protecting all of the sectors on the flash chip.

PARMS: None.
RETURNS: None.

}}

  '// protecting the sectors seems to have a side effect, we are not considering???
  '// we need to read the data sheet more, for now, just return, and don't protect 
  '// all the sectors, this is overkill anyway for now ... 
  return(1)


' // end Flash_Close


PUB Flash_Erase_Chip( wait ) | _spi_data
{{
This function erases the entire chip. The erase command takes a considerable amount
of time, thus this function issues the command and either returns immediately or waits.
To determine if the erase is complete the function polls the status register bit 0
which is the RDY/BSY bit, 0=device is idle, 1=device is busy

PARMS: wait - boolean, 1=wait for erasure to complete, 0-return immediately.

RETURNS: None.

}}
  '// enable SPI interface
   
  '// set CS to SPI select channel 3 (FLASH)
  SPI_Select_Channel( SPI_channel )
  _spi_data := SPI_Write_Read( 8, WRITE_ENABLE, $FF )
   
  '// disable SPI interface
   
  '// set CS to SPI select channel 0 (null)
  SPI_Select_Channel( 0 )

  '// set CS to SPI select channel 3 (FLASH)
  SPI_Select_Channel( SPI_channel )
  _spi_data := SPI_Write_Read( 8, CHIP_ERASE, $FF )
   
  '// disable SPI interface
   
  '// set CS to SPI select channel 0 (null)
  SPI_Select_Channel( 0 )

  ' test if user requested wait for completion?
  if (wait)
    repeat
      ' erase operation still ongoing?
      if ((Flash_Read_Status & $01)==0)
        return (1)
     
  '// return success for now
  return(1)

' /////////////////////////////////////////////////////////////////////////////


PUB Flash_Erase( block ) | _index, _spi_data, _flash_start_addr
{{
This function erases a block of FLASH memory.

PARMS:

block - block address to erase.

RETURNS: None.

}}
  '// STEP 1: enable writing to chip again /////////////////////////////////////////////////////
  '// enable SPI interface
   
  '// set CS to SPI select channel 3 (FLASH)
  SPI_Select_Channel( SPI_channel )
  _spi_data := SPI_Write_Read( 8, WRITE_ENABLE, $FF )
   
  '// disable SPI interface
   
  '// set CS to SPI select channel 0 (null)
  SPI_Select_Channel( 0 )
  Delay_MS( 100 )
   
  '// STEP 2: erase 4K block of chip
  _flash_start_addr := FLASH_BLOCK_TO_ADDR ( block )
   
  '// enable SPI interface
   
  '// set CS to SPI select channel 3 (FLASH)
  SPI_Select_Channel( SPI_channel )
  _spi_data := SPI_Write_Read( 8, BLOCK_ERASE4, $FF )
   
  '// write block address, there are only a handful of blocks since each is 4-64K
  _spi_data := SPI_Write_Read( 8, _flash_start_addr >> 16, $FF )
  _spi_data := SPI_Write_Read( 8, _flash_start_addr >> 8, $FF )
  _spi_data := SPI_Write_Read( 8, _flash_start_addr >> 0, $FF )
   
  '// disable SPI interface
  '// set CS to SPI select channel 0 (null)
  SPI_Select_Channel( 0 )
   
  '// erasure takes a long time... 
  Delay_MS( 500 )
   
  '// return success for now
  return(1)
   
' // end Flash_Erase

' /////////////////////////////////////////////////////////////////////////////

PUB Flash_Write( flash_start_addr, buffer_ptr, num_bytes) | _index, _spi_data
{{
This function writes num_bytes bytes to the FLASH memory at flash_start_addr from the sent buffer_ptr.

It uses the byte/page write function. Although, there is a "sequential" byte write function available
on the AT26DF081A the new rev of the Atel chips (AT25xxxx series) coming out in 2011 have replaced the sequential write with
another function :(, thus we simply have to check for page boundaries and re-issue another page write
every 256 bytes.

PARMS:

flash_start_addr - starting address in the FLASH memory to write the data at.
buffer_ptr       - local byte pointer to Prop memory holding data to write to FLASH memory.
num_bytes        - number of bytes to write to FLASH from source buffer_ptr. 

}}

  '// STEP 1: enable writing to the chip again
  '// enable SPI interface
   
  '// set CS to SPI select channel 3 (FLASH)
  SPI_Select_Channel( SPI_channel )
  _spi_data := SPI_Write_Read( 8, WRITE_ENABLE, $FF )
   
  '// disable SPI interface
   
  '// set CS to SPI select channel 0 (null)
  SPI_Select_Channel( 0 )
   
  '// STEP 2: write buffer to address in flash //////////////////
   
  '// set CS to SPI select channel 3 (FLASH)
  SPI_Select_Channel( SPI_channel )
   
  '// initiate byte/page program command
  _spi_data := SPI_Write_Read( 8, BYTE_PROGRAM, $FF )
   
  '// write starting address byte by byte, MSB first...
  _spi_data := SPI_Write_Read( 8, flash_start_addr >> 16, $FF )
  _spi_data := SPI_Write_Read( 8, flash_start_addr >> 8, $FF )
  _spi_data := SPI_Write_Read( 8, flash_start_addr >> 0, $FF )

  '// write first byte of 
  _spi_data := SPI_Write_Read( 8, byte [buffer_ptr][ 0 ], $FF )

  _index := 1   

  '// write remaining bytes
  repeat until (_index => num_bytes )

    ' now test if we are on a page boundary, if so finish page, and start new page
    if ( ((flash_start_addr + _index) // 256) == 0)
      '// set CS to SPI select channel 0 (null)
      SPI_Select_Channel( 0 )

      ' replace this with polling...
      Delay_MS(10) 

      '// set CS to SPI select channel 3 (FLASH)
      SPI_Select_Channel( SPI_channel )
      _spi_data := SPI_Write_Read( 8, WRITE_ENABLE, $FF )
      SPI_Select_Channel( 0 )

      SPI_Select_Channel( SPI_channel )
               
      '// re-issue byte/page program command
      _spi_data := SPI_Write_Read( 8,BYTE_PROGRAM,$FF )

      '// write starting address byte by byte, MSB first...
      _spi_data := SPI_Write_Read( 8, (flash_start_addr + _index) >> 16, $FF )
      _spi_data := SPI_Write_Read( 8, (flash_start_addr + _index) >> 8, $FF )
      _spi_data := SPI_Write_Read( 8, (flash_start_addr + _index) >> 0, $FF )

      ' end if 

    '// write the next byte of data
    _spi_data := SPI_Write_Read( 8, byte [buffer_ptr][ _index ], $FF )

    _index++
    
    ' // end for index    

  '// set CS to SPI select channel 0 (null)
  SPI_Select_Channel( 0 )

  ' write final bytes left in internal buffer, replace this with polling...
  Delay_MS(10) 
     
  '// disable SPI interface
   
  '// return success
  return(1)
   
' // end Flash_Write

' ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

PUB Flash_Read( flash_start_addr, buffer_ptr, num_bytes) | _index, _spi_data
{{
This function reads num_bytes from the FLASH memory starting at memory location flash_start_addr and stores the data in
buffer_ptr.

PARMS:

flash_start_addr - starting address in FLASH memory to read data from.
buffer_ptr       - local Prop byte pointer to store the read data at.
num_bytes        - the number of bytes to read from the FLASH memory. 

RETURNS: None.
}}

  '// set CS to SPI select channel 3 (FLASH)
  SPI_Select_Channel( SPI_channel )

  '// initiate read command
  _spi_data := SPI_Write_Read( 8, READ_DATA, $FF )

  '// write starting address byte by byte, MSB first...
  _spi_data := SPI_Write_Read( 8, flash_start_addr >> 16, $FF )
  _spi_data := SPI_Write_Read( 8, flash_start_addr >> 8, $FF )
  _spi_data := SPI_Write_Read( 8, flash_start_addr >> 0, $FF )
  
  '// now read data 
  repeat _index from  0 to num_bytes-1
    byte [buffer_ptr][_index] := SPI_Write_Read( 8, $00, $FF )

    ' // end for index

  '// disable SPI interface

  '// set CS to SPI select channel 0 (null)
  SPI_Select_Channel( 0 )

  '// return success 
  return(1)

' // end Flash_Read
PUB Flash_ID (buffer_ptr, num_bytes) | _index, _spi_data
{{
This function reads the JEDEC-ID from the FLASH.

PARMS:

buffer_ptr       - local Prop byte pointer to store the read data at.
num_bytes        - the number of bytes to read from the FLASH memory.

RETURNS: None.
}}

  ' set CS to the selected SPI_channel
  SPI_Select_Channel(SPI_channel)

  ' initiate read JEDEC-ID
  _spi_data := SPI_Write_Read( 8, JEDEC_ID, $FF )

  'read ID
  repeat _index from  0 to num_bytes-1
    byte [buffer_ptr][_index] := SPI_Write_Read( 8, $00, $FF )

    'end for index

  ' disable SPI interface

  ' set CS to SPI select channel 0 (null)
  SPI_Select_Channel( 0 )

  '// return success
  return(1)


' FLASH MACROS

PUB FLASH_ADDR_TO_BLOCK( n )
  n := (n >> 12)

PUB FLASH_BLOCK_TO_ADDR( n )
  n := (n << 12)


' SPI ROUTINES

PUB SPI_Init
{{
This function initializes the SPI IO's, counter, and mux, selects channel 0 and returns.

PARMS: none.
RETURNS: nothing.
}}
  ' set direction to output and low ==> Channel 0
  DIRA[0..2]~~               ' set to output
  OUTA[0..2]         := %000 ' set to LOW

  ' set up SPI lines
  DIRA[SPI_MOSI]     := 1    ' set to output
  DIRA[SPI_MISO]     := 0    ' set to input
  DIRA[SPI_SCK]      := 1    ' set to output

  OUTA[SPI_MOSI]     := 0    ' set to LOW
  OUTA[SPI_SCK]      := 0    ' set to LOW


  ' set SPI to NULL channel 0
  SPI_Select_Channel( 0 )

' end SPI_Init



PUB SPI_Write_Read( num_bits, data_out, bit_mask) | data_in
{{
This function writes and reads SPI data a bit at a time (SPI is a circular buffer protocal), the data is in MSB to LSB format
and up to 32-bits can be transmitted and received, the final result is bit masked by bit_mask

PARMS:

num_bits : number of bits to transmit from data_out
data_out : source of data to transmit
bit_mask : final result of SPI transmission is masked with this to grab the relevant least significant bits

RETURNS: data retrieved from SPI transmission

}}
  ' clear result
  data_in := 0

  ' now read the bits in/out
  repeat num_bits
    ' drop clock
    OUTA[SPI_SCK] := 0

    ' place next bit on MOSI
    OUTA[SPI_MOSI] := ((data_out >> (num_bits-- - 1)) & $01)
 
    ' now read next bit from MISO
    data_in := (data_in << 1) + INA[SPI_MISO]
      
    ' raise clock
    OUTA[SPI_SCK] := 1

  ' set clock and MOSI to LOW on exit
  OUTA[SPI_MOSI]  := 0
  OUTA[SPI_SCK]   := 0

  ' at this point, the data has been written and read, return result
  return ( data_in & bit_mask )            
    
' end SPI_Write_Read


PUB SPI_Select_Channel( channel )
{{
This function sets the active SPI channel chip select on the SPI mux

  SPI_SEL 0 1 2 Channel
          A B C Y
          0 0 0 0
          1 0 0 1
          0 1 0 2
          1 1 0 3
          0 0 1 4
          1 0 1 5
          0 1 1 6
          1 1 1 7
PARMS:

channel : channel 0 to 7 to enable where the channels are defined as follows
          0 - NULL channel, disables all on/off board devices.
          1 - 32K SRAM Bank 0.
          2 - 32K SRAM Bank 1.
          3 - 2MB FLASH Memory Bank 0.
          4 - 2MB FLASH Memory Bank 1.
          5 - SD Card, SD-Modul.
          6 - SD-Modul.
          7 - SD-Modul.

RETURNS: nothing.

}}

  case channel
    0 : outa[0..2] := %000

    1 : outa[0..2] := %100

    2 : outa[0..2] := %010

    3 : outa[0..2] := %110

    4 : outa[0..2] := %001

    5 : outa[0..2] := %101

    6 : outa[0..2] := %011

    7 : outa[0..2] := %111


' end SPI_Select_Channel


' UTILITY ROUTINES ////////////////////////////////////////////////////////////
PUB Get_String(string_ptr, max_length) | _length
{{
This simple function is a single line editor that allows user to enter keys from the keyboard
and then echos them to the screen, when the user hits <ENTER> | <RETURN> the function
exits and returns the string. The function has simple editing and allows <BACKSPACE> to
delete the last character, that's it!

The function outputs to the NTSC terminal

PARMS:

RETURNS: length of string, 0 if NULL.

}}

  ' current length of string buffer
  _length := 0  

  ' draw cursor
  
  repeat 

    ' draw cursor
    lcd.out( "_" )
    lcd.bs
  
    ' wait for keypress 
    repeat while (getTSkey == 0)

    ' user entered a key process it

    ' get key from buffer
    ' g_key := key.key
    g_key := getTSkey

    case g_key
       ASCII_LF, ASCII_CR: ' return    
 
        ' null terminate string and return
        byte [string_ptr][_length] := ASCII_NULL
     
        return( _length )


       ASCII_BS, ASCII_DEL, ASCII_LEFT: ' backspace (edit)

         if (_length > 0)
           ' move cursor back once to overwrite last character on screen
           lcd.out( ASCII_SPACE )
           lcd.bs
           lcd.bs
           
           ' echo character
           lcd.out( ASCII_SPACE )
           lcd.bs
         
           ' decrement length
           _length--
 
       other:    ' all other cases
         ' insert character into string 
         byte [string_ptr][_length] := g_key

         ' update length
         if (_length < max_length )
           _length++
         else
           ' move cursor back once to overwrite last character on screen
           lcd.bs

         ' echo character
         lcd.out( g_key )
     
' // end Get_String

PUB StrCpy( dest_str_ptr, source_str_ptr ) | _index
{{
mimics C strcpy, copies source -> destination including NULL
}}

' test if there is storage
if (dest_str_ptr == NULL)
  return (NULL)

_index := 0

repeat while (byte [ source_str_ptr ][_index] <> NULL)
  ' copy next byte
   byte [ dest_str_ptr ][_index] := byte [ source_str_ptr ][_index]    
   _index++

' null terminate
byte [ dest_str_ptr ][_index] := NULL

' return number of bytes copied
return ( _index )

' // end StrCpy

PUB StrUpper(string_ptr)
{{
upcases a string
}}

  if (string_ptr <> NULL)
    repeat while (byte[ string_ptr ] <> NULL)
      byte[ string_ptr ] :=  ToUpper( byte[ string_ptr ] )
      string_ptr++  

  ' return string
  return (string_ptr)

' // end StrUpper

PUB ToUpper(ch)
{{
returns the uppercase of the sent character
}}

if (ch => $61 and ch =< $7A)
  return(ch-32)
else
  return(ch)

' // end ToUpper

PUB IsInSet(ch, set_string)
{{
tests if sent character is in string
}}

repeat while (byte[set_string][0] <> NULL)
  if (ch == byte[set_string++][0])
    'serial.txstring(string ("found!") )
    return( ch )

'serial.txstring(string ("not found!") )
' not found
return( -1 )    

' // end IsInSet

PUB IsSpace(ch)
{{
tests if sent character is white space, cr, lf, space, tab
}}

if ( (ch == ASCII_SPACE) or (ch == ASCII_LF) or (ch == ASCII_CR) or (ch == ASCII_TAB))
  return (ch)
else
  return(-1)  

' // end IsSpace

PUB IsNull(ch)
{{
tests if sent character is null
}}

if ( (ch == NULL))
  return (1)
else
  return(-1)  

' // end IsNull

PUB IsDigit(ch)
{{
tests if sent character is a number 0..9, returns integer 0..9
}}

if (ch => ASCII_0 and ch =< ASCII_9)
  return (ch-ASCII_0)
else
  return(-1)  

' // end IsDigit

PUB IsAlpha(ch)
{{
tests if sent character is a number a...zA...Z
}}

ch := ToUpper(ch)

if ( (ch => ASCII_A) and (ch =< ASCII_Z))
  return (ch)
else
  return(-1)  

' end IsAlpha

PUB IsPunc(ch)
{{
tests if sent character is a punctuation symbol !@#$%^&*()--+={}[]|\;:'",<.>/?
}}

ch := ToUpper(ch)

if (  ((ch => 33) and (ch =< 47)) or  ((ch => 58) and (ch =< 64)) or ((ch => 91) and (ch =< 96)) or ((ch =>123) and (ch =< 126))  ) 
  return (ch)
else
  return(-1)  

' // end IsPunc
                                                                                
PUB HexToDec(n)
{{
converts hex digit to decimal
}} 
  if ( (n => "0") and (n =< "9") )
    return (n - ASCII_0)
  elseif ( (n => "A") and (n =< "F") )
    return (n - "A" + 10)
  else
    return ( 0 )  

' // end HexToDec


PUB itoa(value, string_ptr) | i
{{
 converts value into a string and stores in string_ptr with NULL termination
 }}

  if value < 0
    -value
    byte [ string_ptr++ ]  := "-"

  i := 1_000_000_000

  repeat 10
    if value => i
      byte [ string_ptr++ ]  := value / i + "0"
      value //= i
      result~~
    elseif result or i == 1
      byte [ string_ptr++ ]  := "0"
    i /= 10

  ' null terminate
  byte [ string_ptr ] := NULL    

' // end itoa

PUB atoi2(string_ptr, length) | _index, _sum, _ch, _sign
{{
this function tries to convert the string to a number, supports binary %, hex $, decimal (default)
string_ptr can be to a null terminated string, or the length can be overridden by sending the length shorter
than the null terminator, ignores ws
Eg. %001010110, $EF, 25
}}

' initialize vars
_index := 0
_sum   := 0
_sign  := 1
 
' consume white space
repeat while (IsSpace( byte[ string_ptr ][_index] ) <> -1)
  _index++
 
' is there a +/- sign?
if (byte [string_ptr][_index] == "+")
  ' consume it
  _index++
elseif (byte [string_ptr][_index] == "-")
  ' consume it
  _index++
  _sign := -1    
     
' try to determine number base
if (byte [string_ptr][_index] == ASCII_HEX)
  _index++
  repeat while ( ( IsDigit(_ch := byte [string_ptr][_index]) <> -1) or ( IsAlpha(_ch := byte [string_ptr][_index]) <> -1)  )
    _index++
    _sum := (_sum << 4) + HexToDec( ToUpper(_ch) )
    if (_index => length)
      return (_sum*_sign)
 
  return(_sum*_sign)
' // end if hex number    
elseif (byte [string_ptr][_index] == ASCII_BIN)
  repeat while ( IsDigit(_ch := byte [string_ptr][++_index]) <> -1)
    _sum := (_sum << 1) + (_ch - ASCII_0)
    if (_index => length)
      return (_sum*_sign)
 
  return(_sum*_sign)
' // end if binary number  
else
  ' must be in default base 10, assume that
  repeat while ( IsDigit(_ch := byte [string_ptr][_index++]) <> -1)
    _sum := (_sum * 10) + (_ch - ASCII_0)
    if (_index => length)
      return (_sum*_sign)
 
  return(_sum*_sign)
 
' else, have no idea of number format!
return(0)
 
' // end atoi2


PUB Delay_MS( time )
{{
Delays "time" milliseconds and returns.
}}

  waitcnt (cnt + time*CLOCKS_PER_MILLISECOND)

' // end Delay_MS


PUB Delay_US( time )
{{
Delays "time" microseconds and returns.

}}

  waitcnt (cnt + time*CLOCKS_PER_MICROSECOND)


' // end Delay_US


DAT

newline_string          byte      $0D, $0A, $00  ' $0D carriage return, $0A line feed

{{ LICENSE SECTION

                          TERMS OF USE: MIT License                                                  |

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation  the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:                                                                   |
                                                                                                                              |
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
}}
