{{ MIO283QT-2 display driver 

Modified and extended by GKR




This software is intended to be used with the MIO283QT-2 display, but may also work with other displays containing the Himax HX8347-D display single chip driver.
Please note, that this software uses the 3 wire serial interface to connect to the HX8347-D. For correct jumper settings consult the HX8347-D datasheet.

Constant names / Theoretical Code Logic derived from Watterott electronic MIO283QT-2 C code / ARM webradio project
SPI Assembly Driver derived from SPI Engine v1.2 by Beau Schwabe (Parallax) and PropTCP by Harrison Pham

History:

  Version 1.0   - initial Version
                - SPI interface in high speed assembler
                - basic graphic routines (circle, rectangle, rectangle filled, pixel, fill screen)
  Version 1.1   - improved fill screen routine (translation into assembler)
                - improved draw pixel routine (translation into assembler)
                - improved power on reset functionality
                - added lcd_drawrow routine for fast picture reproduction

}}

CON

  version = 1     ' major version
  release = 1     ' minor version

CON

  LCD_WIDTH = 320
  LCD_HEIGHT = 240
  DISPLAY_SIZE = LCD_WIDTH * LCD_HEIGHT
  'RGB[r,g,b] = (((r&$F8)<<8)|((g&$FC)<<3)|((b&$F8)>>3))                  '5 red   ,   6 green   ,   5 blue  
  
  LCD_ID = 0      ' change the id according to your jumper settings (must be 1 or 0)
  LCD_DATA = (($72)|(LCD_ID<<2))
  LCD_REGISTER = (($70)|(LCD_ID<<2))

  REGWRITE      = %0_0000_0001
  REGREAD       = %0_0000_0010
  DRAWROW       = %0_0000_0100
  DRAWPIXEL     = %0_0000_1000
  CSON          = %0_0001_0000
  CSOFF         = %0_0010_0000
  INSTROUT      = %0_0100_0000
  DATAOUT       = %0_1000_0000
  LCDFILL       = %1_0000_0000
  w             = 8
  h             = 12
  ' LCD-Color
  tbcolor        = $0000FF
  tfcolor        = $FFFF00
  mbcolor        = $FFFF00
  mfcolor        = $0000FF
  wbcolor        = $FF0000
  wfcolor        = $FFFFFF


VAR
  long col_a
  long row_a
  'long lcdstack[100]


DAT

  cog         long 0
  command     long 0

PUB start(_CS, _SCL, _SDA, _RST) | dummy

  spi_stop

  cspin := |< _cs
  clkpin := |< _scl
  sdapin := |< _sda

  ctramode := %0_00100_00_0000_0000_0000_0000_0000_0000 + _scl
  ctrbmode := %0_00100_00_0000_0000_0000_0000_0000_0000 + _sda
  
  cog := cognew(@init, @command)+1

  delay_ms(5)
   
  reset(_RST)
  clear

PUB fill(color) | i

  area(0, 0, (LCD_WIDTH-1), (LCD_HEIGHT-1))
  drawstart
  setcommand(LCDFILL, @color)
  {repeat i from (LCD_WIDTH*LCD_HEIGHT) to 1
    lcd_draw(color)}
  drawstop

PUB fillrect(x0, y0, x1, y1, color) | wh, tmp

  if x0 > x1
    tmp := x0
    x0 := x1
    x1 := tmp

  if y0 > y1
    tmp := y0
    y0 := y1
    y1 := tmp

  if x1 => LCD_WIDTH or y1 => LCD_HEIGHT
    return

  area(x0, y0, x1, y1)
  drawstart
  repeat wh from (1+(x1-x0))*(1+(y1-y0)) to 1
    draw(color)
  drawstop

  return
  
PUB drawrect(x0, y0, x1, y1, color)

  fillrect(x0, y0, x0, y1, color)
  fillrect(x0, y1, x1, y1, color)
  fillrect(x1, y0, x1, y1, color)
  fillrect(x0, y0, x1, y0, color)

  return

PUB drawcircle(x0, y0, radius, color) | err, x, y

  err := -radius
  x := radius
  y :=0

  area(0, 0, (LCD_WIDTH-1), (LCD_HEIGHT-1))

  repeat while x => y
  
    drawapixel(x0 + x, y0 + y, color)
    drawapixel(x0 - x, y0 + y, color)
    drawapixel(x0 + x, y0 - y, color)
    drawapixel(x0 - x, y0 - y, color)
    drawapixel(x0 + y, y0 + x, color)
    drawapixel(x0 - y, y0 + x, color)
    drawapixel(x0 + y, y0 - x, color)
    drawapixel(x0 - y, y0 - x, color)

    err += y
    y++
    err += y
    if err => 0
      x--
      err -= x
      err -= x
      
  return

PUB fillcircle(x0, y0, radius, color) | err, x, y
  err := -radius
  x   := radius
  y   := 0

  area(0, 0,  (LCD_WIDTH-1), (LCD_HEIGHT-1))

  repeat while x => y
    drawLine(x0 - x, y0 + y, x0 + x, y0 + y, color)
    drawLine(x0 - x, y0 - y, x0 + x, y0 - y, color)
    drawLine(x0 - y, y0 + x, x0 + y, y0 + x, color)
    drawLine(x0 - y, y0 - x, x0 + y, y0 - x, color)

    err += y
    y++
    err += y 
    if err => 0
    
      x-- 
      err -= x 
      err -= x 

  return

PUB drawLine(x0, y0, x1, y1, color) | dx, dy, dx2, dy2, err, stepx, stepy

  if((x0 == x1) or (y0 == y1))    'horizontal or vertical line
  
    fillrect(x0, y0, x1, y1, color)
  
  else
  
    'calculate direction
    dx := x1 - x0
    dy := y1 - y0
    if dx < 0
      dx := -dx
      stepx := -1
    else
      stepx := +1
    if dy < 0
      dy := -dy
      stepy := -1
    else
      stepy := +1
    dx2 := dx << 1
    dy2 := dy << 1
    'draw line
    area(0, 0, (LCD_Width-1), (LCD_Height-1))
    drawapixel(x0, y0, color)
    if  dx > dy
    
      err := dy2 - dx
      repeat while x0 <> x1
      
        if err => 0
        
          y0  += stepy 
          err -= dx2 
         
        x0  += stepx 
        err += dy2 
        drawapixel(x0, y0, color)
       
     
    else
     
      err := dx2 - dy
      repeat while y0 <> y1
       
        if  err => 0
         
          x0  += stepx 
          err -= dy2 
         
        y0  += stepy 
        err += dx2 
        drawapixel(x0, y0, color)

  return 




PUB drawarow(rownumber, pixelperrow, valuechain)

  area(0,rownumber,pixelperrow-1,rownumber)
  setcommand(DRAWROW, @pixelperrow)
  
PUB drawapixel(x, y, color)
' Print a pixel at position x,y on screen
  if x => LCD_WIDTH or y => LCD_HEIGHT
    return
  else
    setcommand(DRAWPIXEL, @x)
    return

PUB draw(color)

  setcommand(DATAOUT, @color)

PUB drawstop | dummy

  setcommand(CSOFF, @dummy)

PUB drawstart | value

  writeregister($22,$00)
  value := LCD_DATA
  setcommand(constant(INSTROUT | CSON), @value)

PUB area(x0,y0,x1,y1)

  writeregister($03, x0)
  writeregister($02, x0>>8)
  writeregister($05, x1)
  writeregister($04, x1>>8)
  writeregister($07, y0)
  writeregister($06, y0>>8)
  writeregister($09, y1)
  writeregister($08, y1>>8) 

PUB cursor(x,y)

  area(x, y, x, y)


PUB Char(col,row,cptr,mode) |bitstat, bitnum,num, x, x0, y, y0, mask, i, j, k, fc, bc
'Subroutine written by GKR
'for 8x8, 8x12,8x16 Fonts == 2,3,4 longs
'col      = colum of character , max 40
'row      = row of charakter   , max 20
'w        = width of font
'h        = hight of font
'charcont
'fc       = foreground color
'bc       = background color
'mode     = 0 text
'         = 1 message
'         = 2 warning
'Set color t = text
'          m = message
'          w = warning
  case mode
    0  : fc := tfcolor
         bc := tbcolor
    1  : fc := mfcolor
         bc := mbcolor
    2  : fc := wfcolor
         bc := wbcolor


  col_a := col
  row_a := row

  if row > LCD_HEIGHT/h
    return
  if col > LCD_WIDTH/w
    return

  x0:= col*w -w                           'number of longs
  y0:= row*h -h
  y := y0
  num := (cptr -32)*3

  repeat i from 0 to ((h/4) - 1) step 1
    'Einlesen der Long-Variable mit 32 bit(0 bis 31)
    bitnum := 31
    mask := FONT8X12[num + i]
    repeat j from 0 to 3 step 1
        x := x0
      repeat k from 0 to (w-1) step 1
        x := x0 + k
        if mask&(|<bitnum) == 0
          drawapixel(x, y, bc)
        else
          drawapixel(x, y, fc)
        bitnum -= 1
      y := y + 1

PUB Strings( col, row, stringptr,mode)
  repeat strsize(stringptr)
    Char(col, row,(byte[stringptr++]), mode)
    col   += 1
    col_a := col
    row_a := row

PUB Text( col, row, stringptr)
  Strings( col, row, stringptr,0)

PUB Warning( col, row, stringptr)
  Strings( col, row, stringptr,2)

PUB Message( col, row, stringptr)
  Strings( col, row, stringptr,1)

PUB setx(col)
  col_a := col

PUB sety(row)
  row_a := row

PUB crlf
'carriage return and line feed
  col_a := 1
  row_a := gety + 1
  if row_a => 20
    row_a := 1

PUB cr
'carriage return
  col_a := 1

PUB bs
'backspace
  col_a -= 1
  if col_a < 1
    col_a := 1

PUB lf
'line feed
  row_a := gety + 1

PUB clear
  fill(tbcolor)
  col_a := 1
  row_a := 1

PUB out(cptr)
  Char(col_a,row_a,cptr,0)
  col_a += 1
  if col_a > 40
    col_a := 1
    row_a += 1
    if row_a > 20
      row_a := 1
      clear

PUB pstring(stringptr)
  Text( col_a, row_a, stringptr)

PUB bin(value, digits)
' Print a binary number

  value <<= 32 - digits
  repeat digits
    out((value <-= 1) & 1 + "0")

PUB getx
  result :=col_a

PUB gety
  result :=row_a

PUB dec(value) | i, z
' Print a decimal number

  if value < 0
    -value
    out("-")

  i := 1_000_000_000
  z~

  repeat 10
    if value => i
      out(value / i + "0")
      value //= i
      z~~
    elseif z or i == 1
      out("0")
    i /= 10


PUB hex(value, digits)
' Print a hexadecimal number

  value <<= (8 - digits) << 2
  repeat digits
    out(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))


PUB reset(_RST)

  dira[_RST]~~
  outa[_RST]~
  delay_ms(50)
  outa[_RST]~~
  delay_ms(50)
  
' driving ability
  writeregister($EA,$00)
  writeregister($EB,$20)
  writeregister($EC,$0C)
  writeregister($ED,$C4)
  writeregister($E8,$40)
  writeregister($E9,$38)
  writeregister($F1,$01)
  writeregister($F2,$10)
  writeregister($27,$A3)

' power voltage
  writeregister($1B,$1B)
  writeregister($1A,$01)
  writeregister($24,$2F)
  writeregister($25,$57)

' VCOM offset
  writeregister($23,$8D)

' power on
  writeregister($18,$36)
  writeregister($19,$01)
  writeregister($01,$00)
  writeregister($1F,$88)
  delay_ms(5)
  writeregister($1F,$80)
  delay_ms(5)
  writeregister($1F,$90)
  delay_ms(5)
  writeregister($1F,$D0)
  delay_ms(5)

' color selection (262K color mode);
' for 65K color mode set value of register $17 to $05 and adjust the number of databits in "data_out_"
  writeregister($17,$06)
  writeregister($36,$00)
  writeregister($28,$38)
  delay_ms(40)      
  writeregister($28,$3C)
  
' display options (no mirror; for mirror set value to h68)
  writeregister($16,$68)

  area(0, 0, (LCD_WIDTH-1), (LCD_HEIGHT-1))

PRI delay_ms(Duration)
  waitcnt(((clkfreq / 1_000 * Duration - 3932)) + cnt)
  
PRI spi_stop
  if cog
    cogstop(cog~ - 1)
  ctra := 0
  command~

PRI writeregister(register, value)

  setcommand(REGWRITE, @register)

PRI setcommand(cmd, argptr)
  command := cmd << 16 + argptr                       'write command and pointer
  repeat while command                                'wait for command to be cleared, signifying receipt


DAT
FONT8X12 long
  long $00000000, $00000000, $00000000 ' $20
  long $00307878, $78303000, $30300000 ' $21
  long $00666666, $24000000, $00000000 ' $22
  long $006C6CFE, $6C6C6CFE, $6C6C0000 ' $23
  long $30307CC0, $C0780C0C, $F8303000 ' $24
  long $000000C4, $CC183060, $CC8C0000 ' $25
  long $0070D8D8, $70FADECC, $DC760000 ' $26
  long $00303030, $60000000, $00000000 ' $27
  long $000C1830, $60606030, $180C0000 ' $28
  long $00603018, $0C0C0C18, $30600000 ' $29
  long $00000066, $3CFF3C66, $00000000 ' $2A
  long $00000018, $187E1818, $00000000 ' $2B
  long $00000000, $00000000, $38386000 ' $2C
  long $00000000, $00FE0000, $00000000 ' $2D
  long $00000000, $00000000, $38380000 ' $2E
  long $00000206, $0C183060, $C0800000 ' $2F
  long $007CC6CE, $DED6F6E6, $C67C0000 ' $30
  long $001030F0, $30303030, $30FC0000 ' $31
  long $0078CCCC, $0C183060, $CCFC0000 ' $32
  long $0078CC0C, $0C380C0C, $CC780000 ' $33
  long $000C1C3C, $6CCCFE0C, $0C1E0000 ' $34
  long $00FCC0C0, $C0F80C0C, $CC780000 ' $35
  long $003860C0, $C0F8CCCC, $CC780000 ' $36
  long $00FEC6C6, $060C1830, $30300000 ' $37
  long $0078CCCC, $CC78CCCC, $CC780000 ' $38
  long $0078CCCC, $CC7C1818, $30700000 ' $39
  long $00000038, $38000038, $38000000 ' $3A
  long $00000038, $38000038, $38183000 ' $3B
  long $000C1830, $60C06030, $180C0000 ' $3C
  long $00000000, $7E007E00, $00000000 ' $3D
  long $00603018, $0C060C18, $30600000 ' $3E
  long $0078CC0C, $18303000, $30300000 ' $3F
  long $007CC6C6, $DEDEDEC0, $C07C0000 ' $40
  long $003078CC, $CCCCFCCC, $CCCC0000 ' $41
  long $00FC6666, $667C6666, $66FC0000 ' $42
  long $003C66C6, $C0C0C0C6, $663C0000 ' $43
  long $00F86C66, $66666666, $6CF80000 ' $44
  long $00FE6260, $647C6460, $62FE0000 ' $45
  long $00FE6662, $647C6460, $60F00000 ' $46
  long $003C66C6, $C0C0CEC6, $663E0000 ' $47
  long $00CCCCCC, $CCFCCCCC, $CCCC0000 ' $48
  long $00783030, $30303030, $30780000 ' $49
  long $001E0C0C, $0C0CCCCC, $CC780000 ' $4A
  long $00E6666C, $6C786C6C, $66E60000 ' $4B
  long $00F06060, $60606266, $66FE0000 ' $4C
  long $00C6EEFE, $FED6C6C6, $C6C60000 ' $4D
  long $00C6C6E6, $F6FEDECE, $C6C60000 ' $4E
  long $00386CC6, $C6C6C6C6, $6C380000 ' $4F
  long $00FC6666, $667C6060, $60F00000 ' $50
  long $00386CC6, $C6C6CEDE, $7C0C1E00 ' $51
  long $00FC6666, $667C6C66, $66E60000 ' $52
  long $0078CCCC, $C07018CC, $CC780000 ' $53
  long $00FCB430, $30303030, $30780000 ' $54
  long $00CCCCCC, $CCCCCCCC, $CC780000 ' $55
  long $00CCCCCC, $CCCCCCCC, $78300000 ' $56
  long $00C6C6C6, $C6D6D66C, $6C6C0000 ' $57
  long $00CCCCCC, $783078CC, $CCCC0000 ' $58
  long $00CCCCCC, $CC783030, $30780000 ' $59
  long $00FECE98, $18306062, $C6FE0000 ' $5A
  long $003C3030, $30303030, $303C0000 ' $5B
  long $000080C0, $6030180C, $06020000 ' $5C
  long $003C0C0C, $0C0C0C0C, $0C3C0000 ' $5D
  long $10386CC6, $00000000, $00000000 ' $5E
  long $00000000, $00000000, $0000FF00 ' $5F
  long $30301800, $00000000, $00000000 ' $60
  long $00000000, $780C7CCC, $CC760000 ' $61
  long $00E06060, $7C666666, $66DC0000 ' $62
  long $00000000, $78CCC0C0, $CC780000 ' $63
  long $001C0C0C, $7CCCCCCC, $CC760000 ' $64
  long $00000000, $78CCFCC0, $CC780000 ' $65
  long $00386C60, $60F86060, $60F00000 ' $66
  long $00000000, $76CCCCCC, $7C0CCC78 ' $67
  long $00E06060, $6C766666, $66E60000 ' $68
  long $00181800, $78181818, $187E0000 ' $69
  long $000C0C00, $3C0C0C0C, $0CCCCC78 ' $6A
  long $00E06060, $666C786C, $66E60000 ' $6B
  long $00781818, $18181818, $187E0000 ' $6C
  long $00000000, $FCD6D6D6, $D6C60000 ' $6D
  long $00000000, $F8CCCCCC, $CCCC0000 ' $6E
  long $00000000, $78CCCCCC, $CC780000 ' $6F
  long $00000000, $DC666666, $667C60F0 ' $70
  long $00000000, $76CCCCCC, $CC7C0C1E ' $71
  long $00000000, $EC6E7660, $60F00000 ' $72
  long $00000000, $78CC6018, $CC780000 ' $73
  long $00002060, $FC606060, $6C380000 ' $74
  long $00000000, $CCCCCCCC, $CC760000 ' $75
  long $00000000, $CCCCCCCC, $78300000 ' $76
  long $00000000, $C6C6D6D6, $6C6C0000 ' $77
  long $00000000, $C66C3838, $6CC60000 ' $78
  long $00000000, $66666666, $3C0C18F0 ' $79
  long $00000000, $FC8C1860, $C4FC0000 ' $7A
  long $001C3030, $60C06030, $301C0000 ' $7B
  long $00181818, $18001818, $18180000 ' $7C
  long $00E03030, $180C1830, $30E00000 ' $7D
  long $0073DACE, $00000000, $00000000 ' $7E
  long $00000010, $386CC6C6, $FE000000 ' $7F


DAT
              org       0
              
init          or        dira, cspin             'pin directions
              or        dira, sdapin
              or        dira, clkpin

              or        outa, cspin             'turn off cs (bring it high)

              mov       frqb, #0                'disable ctrb increment
              mov       ctrb, ctrbmode
              

loop          wrlong    zero,par                'zero command (tell spin we are done processing)
:subloop      rdlong    t1,par wz               'wait for command
        if_z  jmp       #:subloop

              mov       addr, t1                'used for holding return addr to spin vars
        
              rdlong    arg0, t1                'arg0
              add       t1, #4
              rdlong    arg1, t1                'arg1
              add       t1, #4
              rdlong    arg2, t1

              mov       lkup, addr              'get the command var from spin
              shr       lkup, #16               'extract the cmd from the command var

              test      lkup, #CSON wz          'turn on cs
        if_nz andn      outa, cspin
        
              test      lkup, #REGWRITE wz       'spi out
        if_nz call      #reg_write_
              test      lkup, #REGREAD wz        'spi in 
        if_nz call      #reg_read_
              test      lkup, #DRAWROW wz
        if_nz call      #lcd_draw_row_
              test      lkup, #DRAWPIXEL wz
        if_nz call      #lcd_draw_pixel_
              test      lkup, #INSTROUT wz       'spi out
        if_nz call      #instr_out_
              test      lkup, #DATAOUT wz        'spi in 
        if_nz call      #data_out_
              test      lkup, #LCDFILL wz
        if_nz call      #lcd_fill_

              test      lkup, #CSOFF wz         'cs off
        if_nz or        outa, cspin


              jmp       #loop                   ' no cmd found

instr_out_    andn      outa, clkpin
              shl       arg0, #24              
              mov       phsb, arg0              ' data to write
              mov       frqa, freqw             ' 20MHz write frequency
              mov       phsa, #0                ' start at clocking at 0
              
              mov       ctra, ctramode          ' send data @ 20MHz
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              mov       ctra, #0                ' disable
              andn      outa, clkpin
               
instr_out__ret ret


data_out_     shl       arg0, #14

fast_data_out_ andn      outa, clkpin            ' used to send color data              
              mov       phsb, arg0              ' data to write
              mov       frqa, freqw             ' 20MHz write frequency
              mov       phsa, #0                ' start at clocking at 0
              
              mov       ctra, ctramode          ' send 18 bits of data @ 20MHz
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1 
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              mov       ctra, #0                ' disable
              andn      outa, clkpin

fast_data_out__ret              
data_out__ret ret


reg_write_    mov       t2,   arg0
              shl       t2,   #24
              mov       arg0, #LCD_REGISTER
              andn      outa, cspin 
              call      #instr_out_
              andn      outa, clkpin
     
              mov       phsb, t2                ' data to write
              mov       frqa, freqw             ' 20MHz write frequency
              mov       phsa, #0                ' start at clocking at 0
              
              mov       ctra, ctramode          ' send data @ 20MHz
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              mov       ctra, #0                ' disable
              andn      outa, clkpin
              or        outa, cspin

              mov       arg0, #LCD_DATA
              andn      outa, cspin 
              call      #instr_out_
              andn      outa, clkpin
              shl       arg1, #24
      
              mov       phsb, arg1              ' data to write
              mov       frqa, freqw             ' 20MHz write frequency
              mov       phsa, #0                ' start at clocking at 0
              
              mov       ctra, ctramode          ' send data @ 20MHz
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              rol       phsb, #1
              mov       ctra, #0                ' disable
              andn      outa, clkpin
              or        outa, cspin
                
reg_write__ret  ret


reg_read_     
              
reg_read__ret   ret


lcd_fill_     mov       t3, displaysize
              shl       arg0, #14
:loop         call      #fast_data_out_
              djnz      t3, #:loop                               

lcd_fill__ret  ret


lcd_draw_row_  mov    t3, arg0                  ' save number of pixels per row in t3
               mov    t1, arg1                  ' save adress of first pixel data byte
               
               mov    arg0, #$22                ' set register $22 of the lcd to $00 
               mov    arg1, #$00
               call   #reg_write_
               
               andn   outa, cspin               ' enable chip select
               mov    arg0, #LCD_DATA           ' send "data" instruction
               call   #instr_out_
               
:loop         rdbyte arg0, t1                   ' create a long out of 3 bytes of valuechain...
              and    arg0, #$FC                 ' ...using the following pattern:
              shl    arg0, #12                  '((valuechain[x*3+0]&$FC)<<12)|((valuechain[x*3+1]&$FC)<<18)|((valuechain[x*3+2]&$FC)<<24)
              mov    rgb, arg0                  ' where x is the current pixel
              add    t1, #1
              rdbyte arg0, t1
              and    arg0, #$FC
              shl    arg0, #18
              or     rgb, arg0
              add    t1, #1
              rdbyte arg0, t1
              and    arg0, #$FC
              shl    arg0, #24
              or     rgb, arg0
              mov    arg0, rgb                  ' write this long sized value into arg0
                        
              call   #fast_data_out_            ' and pass it on to fast_data_out
              add    t1, #1                     ' increment t1 to get to the first byte of the next pixel
              djnz   t3, #:loop                 ' decrement the number of pixels by one and jump if there are further pixels

              or     outa, cspin                ' otherwise disable chip select

lcd_draw_row__ret  ret                         


lcd_draw_pixel_  mov    t3, arg0
                 mov    t4, arg1
                 mov    arg1, t3
                 mov    arg0, #$03
                 call   #reg_write_
                 mov    arg1, t3
                 mov    arg0, #$05
                 call   #reg_write_
                 shr    t3, #8
                 mov    arg1, t3
                 mov    arg0, #$02
                 call   #reg_write_
                 mov    arg1, t3
                 mov    arg0, #$04
                 call   #reg_write_
                 mov    arg1, t4
                 mov    arg0, #$07
                 call   #reg_write_
                 mov    arg1, t4
                 mov    arg0, #$09
                 call   #reg_write_
                 shr    t4, #8
                 mov    arg1, t4
                 mov    arg0, #$06
                 call   #reg_write_
                 mov    arg1, t4
                 mov    arg0, #$08
                 call   #reg_write_

                 mov    arg0, #$22
                 mov    arg1, #$00
                 call   #reg_write_

                 andn   outa, cspin
                 mov    arg0, #LCD_DATA
                 call   #instr_out_

                 mov    arg0, arg2
                 call   #data_out_
                 or     outa, cspin                 

lcd_draw_pixel__ret     ret

              
zero                    long    0                       'constants

cspin                   long    0                       ' chip select pin
sdapin                  long    0                       ' data pin (prop -> hx8347-d); can be input or output
clkpin                  long    0                       ' clock pin (prop -> hx8347-d)
ctramode                long    0                       ' ctr mode for CLK
ctrbmode                long    0                       ' ctr mode for SPI Out
rgb                     long    0

'freqr                   long    $2000_0000             'frequency of SCK /8 for receive
freqw                   long    $4000_0000              'frequency of SCK /4 for send
phsr                    long    $6000_0000
displaysize             long    DISPLAY_SIZE-1

time                    res     1                       'system counter workspace
                                                        'temp variables
t1                      res     1                       '     loop and cog shutdown
t2                      res     1
t3                      res     1
t4                      res     1

addr                    res     1                       '     Used to hold return address of first Argument passed
lkup                    res     1                       '     Used to hold command lookup

                                                        ' arguments passed to/from high-level Spin
arg0                    long    0                       ' bits / start address
arg1                    long    0                       ' value / count
arg2                    long    0                       ' value     
