{
GBP_SRAM_T7

Testprogram 7 for Gam_Bo_Prop

Testing the SRAM

'Gam_Bo_Prop' is the short name of 'game board propeller'
'Gambo' is also the name given to an unidentified large marine animal

modified 2011 GKR



 based upon

 Propeller C3 Analog to Digital plus SRAM Demo -  c3_ad_sram_demo_010.spin
 AUTHOR: Andre' LaMothe
 LAST MODIFIED: 9/3/10
 VERSION: 1.0
 LICENSE: MIT
 
 COMMENTS: This demo illustrates the C3's on board SRAMs along with the onboard A/D
 converter. The SRAM are implemented with (2) Microchip 23K256 Serial SPI SRAMs. The
 A/D converter is a Microchip MCP3202 dual channel 12-bit converter. This demo takes
 the stand alone A/D demo along with the stand along SRAM demo and merges them into
 something more interesting. Basically, the demo "digitizes" channel 0 A/D port. stores
 it into the SRAMs (12-bit format) and then "plays" the data back.

 This shows off reading the A/D once again, reading and writing the SRAMs, along with
 taking a 12-bit data word from the A/D and spliting it into a pair of 4-bit and 8-bit
 data words and writing them into the SRAMs at the same location across each bank.
 In essence, turning the pair of 32K byte wide SRAMs into a single 32K 16-bit SRAM.

 The 23K256 supports a number of commands,
 we have  only implemented a couple here in these API functions (single byte read/write,
 sequential byte read/write, write status register). The little demo API is shown below
 that implements these:

 PUB SRAM_Init ( ... ) - initializes the SRAMs and sets them to sequential mode.
 PUB SRAM_Write( ... ) - writes bytes of data from a buffer to either SRAM.
 PUB SRAM_Fill( ... )  - fills either SRAM with a constant value. 
 PUB SRAM_Read( ... )  - reads data from either SRAM into a buffer.     
 PUB SRAM64K_MREAD( ... ) - models the pair of 32K SRAMs as a single memory of 64K and reads a byte from it.     
 PUB SRAM64K_MWRITE( ... ) - models the pair of 32K SRAMs as a single memory of 64K and writes a single byte to it.     
 
 However, there are many others you can implement, find out more by reviewing the chip
 specs and data sheets found here:

   http://www.microchip.com/wwwproducts/Devices.aspx?dDocName=en539039
   http://ww1.microchip.com/downloads/en/DeviceDoc/22100E.pdf

 The SRAMs only have 4 commands as shown below here:

 Instruction Name/Value   Description

  READ  0000 0011         Read data from memory array beginning at selected address
  WRITE 0000 0010         Write data to memory array beginning at selected address
  RDSR  0000 0101         Read STATUS register
  WRSR  0000 0001         Write STATUS register

 However, there are 3 modes of operation which are controlled via the status register
 write; byte mode, page mode, and sequential mode -- read the data sheet to learn more.

 The SRAMs are accessed via the SPI bus and are located on SPI channels 1 and 2.
 The SRAMs use a simple SPI command protocal (outlined in the data sheet above)
 that allow single and multiple byte reads and writes.    



}

CON

' PROP CHIP / SYSTEM DEFINES

  ' These settings are for 10 MHZ XTALS
  _clkmode = xtal1 + pll8x          ' enable external clock and pll times 8
  _xinfreq = 10_000_000             ' set frequency to 10 MHZ

  CLOCKS_PER_MICROSECOND = 10*8     ' simple xin*pll / 1_000_000
  CLOCKS_PER_MILLISECOND = 10000*8  ' simple xin*pll / 1_000
  
  _stack   = 125                    ' accomodate display memory and stack

' Gam_Bo_Prop PIN MAP DEFINES

  ' SPI pins
{
  PIN#s direct implemented in subroutine
  SPI_SEL0       = 0  ' used to select SPI channel
  SPI_SEL1       = 1  ' used to select SPI channel
  SPI_SEL2       = 2  ' used to select SPI channel
}

  SPI_MOSI       = 9  ' SPI master out serial in to slave
  SPI_MISO       = 10 ' SPI master in serial out from slave, 10 for good C3, change to 7 for BROKEN C3 with YELLOW jumper.
  SPI_SCK        = 11 ' SPI clock from master to all slaves

  ' SPI channels defines 
  SPI_CHAN_NULL          = 0 ' NULL channel, disables all on/off board devices.
  SPI_CHAN_SRAM0         = 1 ' 32K SRAM Bank 0.
  SPI_CHAN_SRAM1         = 2 ' 32K SRAM Bank 1.
  SPI_CHAN_FLASH0        = 3 ' 2MB FLASH Memory.Bank 0
  SPI_CHAN_FLASH1        = 4 ' 2MB FLASH Memory Bank 1
  SPI_CHAN_SD            = 5 ' SD Card./ SD-ModulChannel 0
  SPI_CHAN_SDM1          = 6 ' SD-Modul Channel 1.
  SPI_CHAN_SDM2          = 7 ' SD-Modul Channel 2.

  ' Detection of SD-Card or SD-Modul
  WP            = 6 ' Detect WP used with CDET to determine, if a SD-Card or SD-Modul is inserted
  CD            = 7 ' Detect CD (Card detect)



  ' Audio stereo output pin define (PWM)
  A_RIGHT       = 4
  A_LEFT        = 5


{
  ' Keyboard

  PIN#s direct implemented in subroutine
  KEY_A         = 12
  KEY_B         = 13
  KEY_C         = 14
  KEY_D         = 15
}

  ' LCD-Display
  LCD_LED       = 8
  LCD_DO        = 18
  LCD_SCK       = 19
  LCD_DI        = 20
  LCD_CS        = 21
  LCD_RES       = 22

  ' Touchscreen controller
  TO_DO         = 23
  TO_SCK        = 24
  TO_DI         = 25
  TO_CS         = 26

  ' I2C pin defines (standard Prop pins)
  SCL           = 28
  SDA           = 29
 
  ' TX/RX pin defines (standard Prop pins)
  RXD           = 30
  TXD           = 31


{
  'Unused and unconnected Pins
  P3            = 3
  P16           = 16
  P17           = 17
  P27           = 27
}
  ' ASCII codes for ease of parser development
  ASCII_A      = 65
  ASCII_B      = 66
  ASCII_C      = 67
  ASCII_D      = 68
  ASCII_E      = 69
  ASCII_F      = 70
  ASCII_G      = 71
  ASCII_H      = 72
  ASCII_O      = 79  
  ASCII_P      = 80
  ASCII_Z      = 90
  ASCII_0      = 48
  ASCII_9      = 57
  ASCII_LEFT   = $C0
  ASCII_RIGHT  = $C1
  ASCII_UP     = $C2
  ASCII_DOWN   = $C3 
  ASCII_BS     = $C8 ' backspace
  ASCII_DEL    = $C9 ' delete
  ASCII_LF     = $0A ' line feed 
  ASCII_CR     = $0D ' carriage return
  ASCII_ESC    = $CB ' escape
  ASCII_HEX    = $24 ' $ for hex
  ASCII_BIN    = $25 ' % for binary
  ASCII_LB     = $5B ' [ 
  ASCII_SEMI   = $3A ' ; 
  ASCII_EQUALS = $3D ' = 
  ASCII_PERIOD = $2E ' .
  ASCII_COMMA  = $2C ' ,
  ASCII_SHARP  = $23 ' #
  ASCII_NULL   = $00 ' null character
  ASCII_SPACE  = $20 ' space
  ASCII_TAB    = $09 ' horizontal tab



  ' SRAM function helper constants
  SPI_MODE_BYTE       = %00_00000_1
  SPI_MODE_PAGE       = %10_00000_1
  SPI_MODE_SEQUENTIAL = %01_00000_1
  
  ' null pointer, null character
  NULL         = 0

  LCD_WIDTH  = 320
  LCD_HEIGHT = 240

  ' LCD-Color
  tbcolor        = $0000FF
  tfcolor        = $FFFF00
  mbcolor        = $FF0000
  mfcolor        = $0000FF
  wbcolor        = $FF0000
  wfcolor        = $FFFFFF

VAR

  ' generic globals to make demo writing easy
  long g_data, g_data2, g_status    
  long g_data32, g_data16, g_spi_result
  long g_key, g_temp1, g_temp2, g_temp3, g_index, g_length
  long g_tempx, g_tempy 
  byte wbuffer,rbuffer,writep,cardd



OBJ

  ' current drivers used in this version of SPI interface 
  key   : "GBP_gkey.spin"          ' keyboard driver
  lcd   : "GBP_MIO283QT2.spin"    ' LCD driver

DAT

  sram_buffer byte 1,4,9,16,25,36,49,64,81,100,121,144,169,196,225


PUB Main : status



  dira[8]~~      'LCD backlight
  outa[8]~~


  ' let the system initialize 
  'Delay_US( 1*1_000_000 )

  ' initialize the LCD-Display
  lcd.start(LCD_CS, LCD_SCK, LCD_DI, LCD_RES)

  ' initialize the Keyboard
  key.start

  ' delay a moment
  'Delay_US(1*1_000_000)
  



  ' initialize SPI IO Bus
  SPI_Init

  ' initialize the SRAMs into sequential access mode
  'SRAM_Init( SPI_MODE_SEQUENTIAL )


  ' enter infinite test loop
  repeat
    ' clear screen
    lcd.fill(tbcolor)

    ' call the test
    SRAM_Test
     
' end Main

PUB SRAM_Test | channel, addr, err, char, keycode, status

  'Detection of SD-Card, SD-Modul or nothing inserted wit WP and CD
   SDM_Detect

  'SRAM Test

  'This test write value $FF from address 0 to 32767 to each SRAM if you
  'press the korresponding key

  lcd.Text( 3, 1, string ("Gam_Bo_Prop SRAM Test"))
  lcd.Text( 3, 3, string ("Key 1 = SRAM on channel 1"))
  lcd.Text( 3, 4, string ("Key 2 = SRAM on channel 2"))
  lcd.Text( 3, 5, string ("If available on SD-Module :"))
  lcd.Text( 3, 6, string ("Key 5 = SRAM on channel 5"))
  lcd.Text( 3, 7, string ("Key 6 = SRAM on channel 6"))
  lcd.Text( 3, 8, string ("Key 7 = SRAM on channel 7"))
  lcd.Text( 3, 9, string ("Press key 1,2,5,6 or 7"))


  ' wait for keypress (button 1 to 8)
  keycode := key.waitforkey
  case keycode
    1 : channel := 1
    2 : channel := 2
    5 : channel := 5
    6 : channel := 6
    7 : channel := 7
    other : keycode := 0

  ' initialize the SRAMs into sequential access mode
  SRAM_Init( channel, SPI_MODE_SEQUENTIAL )
  ' Read Statusregister
  ' Select channel
  SPI_Select_Channel( channel )
  ' set SRAM to sequential access mode (see page 6 of data sheet)
  ' read status register
  SPI_Write_Read( 8, %00000101  , $FF )
  status := 0
  status := SPI_Write_Read( 8, $FF , $FF )
  ' If status = %00000000 the SRAM ist not available a new test begins
  ' The HOLD-bit in status-register is set to 1 ever.
  if status == %00000000
    'Print a message
    lcd.Warning( 3, 10, string ("SRAM not available       "))
    Delay_US(1*1_000_000)
    ' and finally de-select all SPI devices
    SPI_Select_Channel( 0 )
    'reset keycode
    keycode := 0
  else
  ' initialize the SRAMs into sequential access mode
    itoa (channel, char)
    lcd.Text( 3, 10, string ("Initialize SRAM"))
    lcd.Text(20, 10, char)
    SRAM_Init( channel, SPI_MODE_SEQUENTIAL )


    lcd.Text( 3, 11, string ("Write $FF to SRAM "))
    wbuffer := $FF

    repeat addr from 0 to 32767
      SRAM_Write( channel, addr, 1, @wbuffer)

    lcd.Text( 30, 11, string ("Ready"))
    lcd.Text( 3, 12, string ("Read SRAM and compare "))

    err := 0
    repeat addr from 0 to 32767
      rbuffer := $00
      SRAM_Read( channel , addr, 1, @rbuffer)
      if rbuffer <> $FF
        err += 1

    lcd.Text( 30, 12, string ("Ready"))
    itoa (err, char)
    lcd.Text( 3, 13, string ("Error(s)"))
    lcd.Text(30, 13, char)

    lcd.Text( 3, 14, string ("Write $00 to SRAM "))
    wbuffer := $00

    repeat addr from 0 to 32767
      SRAM_Write( channel, addr, 1, @wbuffer)

    lcd.Text( 30, 14, string ("Ready"))

    lcd.Text( 3, 15, string ("Read SRAM and compare "))

    err := 0

    repeat addr from 0 to 32767
      rbuffer := $FF
      SRAM_Read( channel , addr, 1, @rbuffer)
      if rbuffer <> $00
        err += 1

    lcd.Text( 30, 15, string ("Ready"))
    itoa (err, char)
    lcd.Text( 3, 16, string ("Error(s)"))
    lcd.Text(30, 16, char)
    lcd.Text( 3, 17, string ("Press any key to run new test...."))

    ' wait for key
    key.waitforkey

' end SRAM_Test

' SRAM API


PUB SRAM_Init (channel, mode ) | _spi_word
{{
This function initializes both banks of the Microchip 23K256 SRAMs
serial SPI SRAMs'. The initialization simply sets the STATUS register
of each SRAM into "byte", "page" or "sequential" read/write mode to allow more
efficient access of the SRAMs. The SRAM API currently relies on "sequential"
mode, so if yoo want to use them, this function should be called with SPI_MODE_SEQUENTIAL

The 23K256 supports a number of commands, we have
only implemented a couple here in these API functions (single byte read/write,
multiple byte read/write). However, there are many others you can implement, find
out more by reviewing the chip specs and data sheets found here:

http://www.microchip.com/wwwproducts/Devices.aspx?dDocName=en539039
http://ww1.microchip.com/downloads/en/DeviceDoc/22100E.pdf

PARMS:  mode - %00_00000_1 = SPI_MODE_BYTE
               %10_00000_1 = SPI_MODE_PAGE
               %01_00000_1 = SPI_MODE_SEQUENTIAL

RETURNS: Nothing. 

}}

  ' first selected SRAM
  ' channel 1 and 2 are on the board and
  ' 5, 6 and 7 are possible on a SD-module.
  SPI_Select_Channel( channel )
  ' set SRAM to sequential access mode (see page 6 of data sheet)
  ' write status register command "sequential access mode" along with disable
  ' "HOLD" to setup SRAM for sequential reading/writing
  SPI_Write_Read( 16, %0000000_1 << 8 |  mode , $FF )

  ' and finally de-select all SPI devices
  SPI_Select_Channel( 0 )

' end SRAM_Init

PUB SRAM_Write( channel, addr, num_bytes, src_buffer_ptr) | _index, _data, _spi_word
{{
This function writes a number of bytes to either bank of the Microchip 23K256
serial SPI SRAMs' from a buffer. The 23K256 supports a number of commands, we have
only implemented a couple here in these API functions (single byte read/write,
multiple byte read/write). However, there are many others you can implement, find
out more by reviewing the chip specs and data sheets found here:

http://www.microchip.com/wwwproducts/Devices.aspx?dDocName=en539039
http://ww1.microchip.com/downloads/en/DeviceDoc/22100E.pdf


PARMS:  channel        - [0,1] the SRAM bank to write to, there are two banks, each holds 32K bytes
        addr           - address in SRAM to write to [0...32767]
        num_bytes      - number of bytes to write from source buffer to SRAM    
        src_buffer_ptr - pointer to the buffer of bytes to write to SRAM

RETURNS: number of bytes written 

}}

  ' test if there is anything to write?
  if (num_bytes => 1)

    ' select SPI channel for requested SRAM bank
    SPI_Select_Channel( channel )

    ' set instruction word ( write sequential byte:8 | address:16 )
    _spi_word := (%00000010 << 16) + (addr)

    ' and now send command to SRAM for sequential writing
    SPI_Write_Read( 24, _spi_word, $FF )
    
    ' write the bytes into the SRAM
    repeat _index from 0 to num_bytes-1

      ' get byte to write
      _data := byte[ src_buffer_ptr][ _index ]
       
      ' write the byte
      SPI_Write_Read( 8, _data, $FF )

    ' de-select SRAM SPI channel
    SPI_Select_Channel( 0 )

    ' return bytes written
    return( num_bytes )

  else
    ' catch error
    return 0 

' end SRAM_Write

PUB SRAM_Fill( channel, addr, num_bytes, value) | _index, _data, _spi_word
{{
This function writes a constant value to either bank of the Microchip 23K256
serial SPI SRAMs' from a buffer. The 23K256 supports a number of commands, we have
only implemented a couple here in these API functions (single byte read/write,
multiple byte read/write). However, there are many others you can implement, find
out more by reviewing the chip specs and data sheets found here:

http://www.microchip.com/wwwproducts/Devices.aspx?dDocName=en539039
http://ww1.microchip.com/downloads/en/DeviceDoc/22100E.pdf


PARMS:  channel        - [1,2,5,6,7] the SRAM SPI-channel, each SRAM holds 32K bytes
        addr           - address in SRAM to write to [0...32767]
        num_bytes      - number of bytes to write from source buffer to SRAM    
        value          - byte value to write to SRAM

RETURNS: number of bytes written 

}}

  ' test if there is anything to fill?
  if (num_bytes => 1)

    ' select SPI channel for requested SRAM channel
    SPI_Select_Channel( channel )

    ' set instruction word ( write byte sequential:8 | address:16 )
    _spi_word := (%00000010 << 16) + (addr)

    ' setup for sequential byte write, this includes the command and address only
    SPI_Write_Read( 24, _spi_word, $FF )

    ' now the chip is ready to receive bytes (as many as we want to send)
    repeat _index from 0 to num_bytes-1
      SPI_Write_Read( 8, value, $FF )

    ' de-select SRAM channel
    SPI_Select_Channel( 0 )

    ' return bytes written
    return( num_bytes )

  else
    ' catch error
    return 0 

' end SRAM_Fill

PUB SRAM_Read(channel, addr, num_bytes, dest_buffer_ptr) | _index, _data, _spi_word
{{

This function reads a number of bytes from either bank of the SRAM into a buffer.

The 23K256 supports a number of commands, we have only implemented a couple here
in these API functions (single byte read/write, multiple byte read/write).
However, there are many others you can implement, find out more by reviewing
the chip specs and data sheets found here:

http://www.microchip.com/wwwproducts/Devices.aspx?dDocName=en539039
http://ww1.microchip.com/downloads/en/DeviceDoc/22100E.pdf

PARMS:  bank            - [0,1] the SRAM bank to read from, there are two banks, each holds 32K bytes
        addr            - address in SRAM to read from [0..32767]
        num_bytes       - number of bytes to read from SRAM into destination buffer
        dest_buffer_ptr - pointer to the buffer of bytes to read SRAM into

RETURNS: number of bytes read 

}}

  ' test for anything to read?
  if (num_bytes => 1)

    ' select SPI channel for requested SRAM bank
    SPI_Select_Channel( channel )

    ' set instruction word ( read byte sequential:8 | address:16 )
    _spi_word := (%00000011 << 16) + (addr)

    ' setup for sequential byte read, this includes the command and address only
    SPI_Write_Read( 24, _spi_word, $FF )

    ' now the chip is ready to be read from
    repeat _index from 0 to num_bytes-1

      ' read next byte from SRAM into receiving buffer
      byte [dest_buffer_ptr][_index] := SPI_Write_Read( 8, $00, $FF )

    ' de-select SRAM channel
    SPI_Select_Channel( 0 )
    
    ' return bytes read
    return( num_bytes )
  else
    ' catch error
    return 0 

' end SRAM_Read


{{
PUB SRAM64K_MREAD( addr ) | _spi_word, _data
' Doppelklammer
This function reads a single byte from either bank of the Microchip 23K256
serial SPI SRAMs'. This function is different from the SRAM_Read function
in that it accesses the two banks of SRAM as a contiguous region of 64K bytes
for you, and thus can be though of as a crude "array" access macro, rather
than a function. Ultimately, this should be converted to ASM along with
all the other important peripheral access methods.

The 23K256 supports a number of commands, we have
only implemented a couple here in these API functions (single byte read/write,
multiple byte read/write). However, there are many others you can implement, find
out more by reviewing the chip specs and data sheets found here:

http://www.microchip.com/wwwproducts/Devices.aspx?dDocName=en539039
http://ww1.microchip.com/downloads/en/DeviceDoc/22100E.pdf


PARMS:  addr - address in SRAMs to write to [0...65535].
    
RETURNS: The 8-bit data from the SRAM. 

'Doppelklammer ende

    ' the only tricky thing here is we need to take an address 0..65535 and then
    ' decide which bank it refers to and which byte in that bank, basically, some
    ' bit masking and shifts to accomplish this

    ' select SPI channel for requested SRAM bank
    SPI_Select_Channel( SPI_CHAN_SRAM0 + (addr >> 15) )

    ' set instruction word ( read byte sequential:8 | address:16 | dummy:8 )
    _spi_word := (%00000011 << 24) + (addr << 8) + 0 

    ' read the byte of data 
    _data := SPI_Write_Read( 32, _spi_word, $FF )

    ' de-select SRAM SPI channel
    SPI_Select_Channel( 0 )

    ' return data
    return ( _data )
    
' end SRAM64K_READ



PUB SRAM64K_MWRITE( addr, data ) | _spi_word
'double brace
This function writes a single byte to either bank of the Microchip 23K256
serial SPI SRAMs'. This function is different from the SRAM_Write|Fill functions
in that it accesses the two banks of SRAM as a contiguous region of 64K bytes
for you, and thus can be though of as a crude "array" access macro, rather
than a function. Ultimately, this should be converted to ASM along with
all the other important peripheral access methods.

The 23K256 supports a number of commands, we have
only implemented a couple here in these API functions (single byte read/write,
multiple byte read/write). However, there are many others you can implement, find
out more by reviewing the chip specs and data sheets found here:

http://www.microchip.com/wwwproducts/Devices.aspx?dDocName=en539039
http://ww1.microchip.com/downloads/en/DeviceDoc/22100E.pdf


PARMS:  addr - address in SRAMs to write to [0...65535].
        data - the byte to write.    

RETURNS: Nothing. 

' double brace end

    ' the only tricky thing here is we need to take an address 0..65535 and then
    ' decide which bank it refers to and which byte in that bank, basically, some
    ' bit masking and shifts to accomplish this

    ' select SPI channel for requested SRAM bank
    SPI_Select_Channel( SPI_CHAN_SRAM0 + (addr >> 15) )

    ' set instruction word ( write sequential byte:8 | address:16 | data: 8)
    _spi_word := (%00000010 << 24) + ( (addr & $7FFF) << 8) + data

    ' write the byte of data 
    SPI_Write_Read( 32, _spi_word, $FF )

    ' de-select SRAM SPI channel
    SPI_Select_Channel( 0 )

' end SRAM64K_WRITE
}}

PUB SPI_Init
{{
This function initializes the SPI IO's.

PARMS: none.
RETURNS: nothing.
}}
  ' set direction to output and low ==> Channel 0
  DIRA[0..2]~~               ' set to output
  OUTA[0..2]         := %000 ' set low

  ' set up SPI lines
  DIRA[SPI_MOSI]     := 1    ' set to output
  DIRA[SPI_MISO]     := 0    ' set to input
  DIRA[SPI_SCK]      := 1    ' set to output

  OUTA[SPI_MOSI]     := 0    ' set to LOW
  OUTA[SPI_SCK]      := 0    ' set to LOW


  ' set SPI to NULL channel 0
  SPI_Select_Channel( 0 )

' end SPI_Init

PUB SPI_Write_Read( num_bits, data_out, bit_mask) | data_in
{{
This function writes and reads SPI data a bit at a time (SPI is a circular buffer protocal), the data is in MSB to LSB format
and up to 32-bits can be transmitted and received, the final result is bit masked by bit_mask

PARMS:

num_bits : number of bits to transmit from data_out
data_out : source of data to transmit
bit_mask : final result of SPI transmission is masked with this to grab the relevant least significant bits

RETURNS: data retrieved from SPI transmission

}}
  ' clear result
  data_in := 0

  ' now read the bits in/out
  repeat num_bits
    ' drop clock
    OUTA[SPI_SCK] := 0

    ' place next bit on MOSI
    OUTA[SPI_MOSI] := ((data_out >> (num_bits-- - 1)) & $01)
 
    ' now read next bit from MISO
    data_in := (data_in << 1) + INA[SPI_MISO]
      
    ' raise clock
    OUTA[SPI_SCK] := 1

  ' set clock and MOSI to LOW on exit
  OUTA[SPI_MOSI]  := 0
  OUTA[SPI_SCK]   := 0

  ' at this point, the data has been written and read, return result
  return ( data_in & bit_mask )            
    
' end SPI_Write_Read


PUB SDM_Detect
'This function detect a SD-Card or Modul in the SD-Connector
'and print the SD-Status in Line 1 Position 33 to 40

  dira[WP]~
  dira[CD]~
  writep := ina[WP]
  cardd  := ina[CD]
  if writep == 0 and cardd == 1
    lcd.Message(33, 1, string("SD-Modul"))
  elseif (writep == 1) and (cardd == 1)
    lcd.Message(33, 1, string("empty   "))
  else
    lcd.Message(33, 1, string("SD-Card "))


PUB SPI_Select_Channel( channel )
{{
This function sets the active SPI channel chip select on the SPI mux

  SPI_SEL 0 1 2 Channel
          A B C Y
          0 0 0 0
          1 0 0 1
          0 1 0 2
          1 1 0 3
          0 0 1 4
          1 0 1 5
          0 1 1 6
          1 1 1 7
PARMS:

channel : channel 0 to 7 to enable where the channels are defined as follows
          0 - NULL channel, disables all on/off board devices.
          1 - 32K SRAM Bank 0.
          2 - 32K SRAM Bank 1.
          3 - 2MB FLASH Memory Bank 0.
          4 - 2MB FLASH Memory Bank 1.
          5 - SD Card, SD-Modul.
          6 - SD-Modul.
          7 - SD-Modul.

RETURNS: nothing.

}}

  case channel
    0 : outa[0..2] := %000

    1 : outa[0..2] := %100

    2 : outa[0..2] := %010

    3 : outa[0..2] := %110

    4 : outa[0..2] := %001

    5 : outa[0..2] := %101

    6 : outa[0..2] := %011

    7 : outa[0..2] := %111


' end SPI_Select_Channel


' UTILITY ROUTINES ////////////////////////////////////////////////////////////
{
PUB Get_String(string_ptr, max_length) | _length
{{
This simple function is a single line editor that allows user to enter keys from the keyboard
and then echos them to the screen, when the user hits <ENTER> | <RETURN> the function
exits and returns the string. The function has simple editing and allows <BACKSPACE> to
delete the last character, that's it!

The function outputs to the NTSC terminal

PARMS:

RETURNS: length of string, 0 if NULL.

}}

  ' current length of string buffer
  _length := 0  

  ' draw cursor
  
  repeat 

    ' draw cursor
    gfx_ntsc.Out_Term( "_" )
    gfx_ntsc.Out_Term( $08 )
  
    ' wait for keypress 
    repeat while (kbd.gotkey == FALSE)

    ' user entered a key process it

    ' get key from buffer
    g_key := kbd.key
     
    case g_key
       ASCII_LF, ASCII_CR: ' return    
 
        ' null terminate string and return
        byte [string_ptr][_length] := ASCII_NULL
     
        return( _length )


       ASCII_BS, ASCII_DEL, ASCII_LEFT: ' backspace (edit)

         if (_length > 0)
           ' move cursor back once to overwrite last character on screen
           gfx_ntsc.Out_Term( ASCII_SPACE )
           gfx_ntsc.Out_Term( $08 )          
           gfx_ntsc.Out_Term( $08 )
           
           ' echo character
           gfx_ntsc.Out_Term( ASCII_SPACE )
           gfx_ntsc.Out_Term( $08 )
         
           ' decrement length
           _length--
 
       other:    ' all other cases
         ' insert character into string 
         byte [string_ptr][_length] := g_key

         ' update length
         if (_length < max_length )
           _length++
         else
           ' move cursor back once to overwrite last character on screen
           gfx_ntsc.Out_Term( $08 )          

         ' echo character
         gfx_ntsc.Out_Term( g_key )
     
' // end Get_String


}

PUB StrCpy( dest_str_ptr, source_str_ptr ) | _index
{{
mimics C strcpy, copies source -> destination including NULL
}}

' test if there is storage
if (dest_str_ptr == NULL)
  return (NULL)

_index := 0

repeat while (byte [ source_str_ptr ][_index] <> NULL)
  ' copy next byte
   byte [ dest_str_ptr ][_index] := byte [ source_str_ptr ][_index]    
   _index++

' null terminate
byte [ dest_str_ptr ][_index] := NULL

' return number of bytes copied
return ( _index )

' // end StrCpy

PUB StrUpper(string_ptr)
{{
upcases a string
}}

  if (string_ptr <> NULL)
    repeat while (byte[ string_ptr ] <> NULL)
      byte[ string_ptr ] :=  ToUpper( byte[ string_ptr ] )
      string_ptr++  

  ' return string
  return (string_ptr)

' // end StrUpper

PUB ToUpper(ch)
{{
returns the uppercase of the sent character
}}

if (ch => $61 and ch =< $7A)
  return(ch-32)
else
  return(ch)

' // end ToUpper

PUB IsInSet(ch, set_string)
{{
tests if sent character is in string
}}

repeat while (byte[set_string][0] <> NULL)
  if (ch == byte[set_string++][0])
    'serial.txstring(string ("found!") )
    return( ch )

'serial.txstring(string ("not found!") )
' not found
return( -1 )    

' // end IsInSet

PUB IsSpace(ch)
{{
tests if sent character is white space, cr, lf, space, tab
}}

if ( (ch == ASCII_SPACE) or (ch == ASCII_LF) or (ch == ASCII_CR) or (ch == ASCII_TAB))
  return (ch)
else
  return(-1)  

' // end IsSpace

PUB IsNull(ch)
{{
tests if sent character is null
}}

if ( (ch == NULL))
  return (1)
else
  return(-1)  

' // end IsNull

PUB IsDigit(ch)
{{
tests if sent character is a number 0..9, returns integer 0..9
}}

if (ch => ASCII_0 and ch =< ASCII_9)
  return (ch-ASCII_0)
else
  return(-1)  

' // end IsDigit

PUB IsAlpha(ch)
{{
tests if sent character is a number a...zA...Z
}}

ch := ToUpper(ch)

if ( (ch => ASCII_A) and (ch =< ASCII_Z))
  return (ch)
else
  return(-1)  

' end IsAlpha

PUB IsPunc(ch)
{{
tests if sent character is a punctuation symbol !@#$%^&*()--+={}[]|\;:'",<.>/?
}}

ch := ToUpper(ch)

if (  ((ch => 33) and (ch =< 47)) or  ((ch => 58) and (ch =< 64)) or ((ch => 91) and (ch =< 96)) or ((ch =>123) and (ch =< 126))  ) 
  return (ch)
else
  return(-1)  

' // end IsPunc

PUB HexToDec(n)
{{
converts hex digit to decimal
}} 
  if ( (n => "0") and (n =< "9") )
    return (n - ASCII_0)
  elseif ( (n => "A") and (n =< "F") )
    return (n - "A" + 10)
  else
    return ( 0 )  

' // end HexToDec

PUB itoa(value, string_ptr) | i
{{
 converts value into a string and stores in string_ptr with NULL termination
 }}

  if value < 0
    -value
    byte [ string_ptr++ ]  := "-"

  i := 1_000_000_000

  repeat 10
    if value => i
      byte [ string_ptr++ ]  := value / i + "0"
      value //= i
      result~~
    elseif result or i == 1
      byte [ string_ptr++ ]  := "0"
    i /= 10

  ' null terminate
  byte [ string_ptr ] := NULL    

' // end itoa

PUB atoi2(string_ptr, length) | _index, _sum, _ch, _sign
{{
this function tries to convert the string to a number, supports binary %, hex $, decimal (default)
string_ptr can be to a null terminated string, or the length can be overridden by sending the length shorter
than the null terminator, ignores ws
Eg. %001010110, $EF, 25
}}

' initialize vars
_index := 0
_sum   := 0
_sign  := 1
 
' consume white space
repeat while (IsSpace( byte[ string_ptr ][_index] ) <> -1)
  _index++
 
' is there a +/- sign?
if (byte [string_ptr][_index] == "+")
  ' consume it
  _index++
elseif (byte [string_ptr][_index] == "-")
  ' consume it
  _index++
  _sign := -1    
     
' try to determine number base
if (byte [string_ptr][_index] == ASCII_HEX)
  _index++
  repeat while ( ( IsDigit(_ch := byte [string_ptr][_index]) <> -1) or ( IsAlpha(_ch := byte [string_ptr][_index]) <> -1)  )
    _index++
    _sum := (_sum << 4) + HexToDec( ToUpper(_ch) )
    if (_index => length)
      return (_sum*_sign)
 
  return(_sum*_sign)
' // end if hex number    
elseif (byte [string_ptr][_index] == ASCII_BIN)
  repeat while ( IsDigit(_ch := byte [string_ptr][++_index]) <> -1)
    _sum := (_sum << 1) + (_ch - ASCII_0)
    if (_index => length)
      return (_sum*_sign)
 
  return(_sum*_sign)
' // end if binary number  
else
  ' must be in default base 10, assume that
  repeat while ( IsDigit(_ch := byte [string_ptr][_index++]) <> -1)
    _sum := (_sum * 10) + (_ch - ASCII_0)
    if (_index => length)
      return (_sum*_sign)
 
  return(_sum*_sign)
 
' else, have no idea of number format!
return(0)
 
' // end atoi2

PUB Delay_MS( time )
{{
Delays "time" milliseconds and returns.
}}

  waitcnt (cnt + time*CLOCKS_PER_MILLISECOND)

' // end Delay_MS


PUB Delay_US( time )
{{
Delays "time" microseconds and returns.

}}

  waitcnt (cnt + time*CLOCKS_PER_MICROSECOND)


' // end Delay_US


DAT

newline_string          byte      $0D, $0A, $00  ' $0D carriage return, $0A line feed





{{ LICENSE SECTION

                          TERMS OF USE: MIT License                                                  |

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation  the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:                                                                   |
                                                                                                                              |
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
}}
